
#ifndef L1394_THREAD_HPP
#define L1394_THREAD_HPP

#include <pthread.h>

#include <signal.h>

namespace L1394 {
  class ThreadCondition;

  /** This class is a wrapper for the pthread_mutex variables.
   */
  class ThreadMutex {
    /*Define ThreadCondiotion as friend. So it can access <mutex>*/
    friend class ThreadCondition;
  public:
    enum MutexKind {FAST, RECURSIVE};

    /** Constructor
     * \param MutexKind : This parameter determines whether a ThreadMutex can be
     *        locked again by a thread that already owns it (Recursive) or not (Fast).
     *        Default value is Fast.
     */
    ThreadMutex(MutexKind mutex_kind = FAST);

    /** Destructor
     */
    ~ThreadMutex();

    /** This method locks the mutex. If the mutex is currently unlocked, it becomes locked
     * and  owned  by  the calling  thread. If the mutex is already locked by
     * another thread, the mutex suspends the calling thread until the mutex is unlocked.
     * If the MutexKind is 'Recursive' and the current thread locks again, a counter
     * is increased.
     */	
    void lock()      const;

    /** This method unlocks the mutex. If the MutexKind 'Fast', unlock
     * releases the mutex. If MutexKind is 'Recursive', it
     * decrements the locking  count  of  the  mutex and only when this
     * count reaches zero the mutex is released.
     */
    void unlock()    const;

    /** This method differs form lock, that this method returns imediatly.
     * Return value is FAILURE if the mutex is already locked by another thread
     * else SUCCESS
     */
    int tryLock() const;

  private:
    
    ThreadMutex(const ThreadMutex&);    //disabled copy-constructor
    void operator=(const ThreadMutex&); //disabled assignement operator

    mutable int lock_counter;
    MutexKind mutex_kind;
    mutable pthread_mutex_t mutex;
    mutable pthread_t current_thread;
  };


  /** This class can be used to ensure that a locked mutex is unlocked if
   * you leave the scope
   */
  class MutexGuard {
  public:
    /** Constructor
     * Locks the mutex
     */
    MutexGuard(const ThreadMutex& mutex) : mutex(mutex) { is_owner = false; lock(); }

    /** Destructor
     * Unlocks the Mutex, if it is not unlocked before
     */
    ~MutexGuard() {mutex.unlock();}

    /** This method unlocks the Mutex, if this MutexGuard is the owner
      */
    void unlock()     const  { if (is_owner) { mutex.unlock(); is_owner = false; } }

    /** This method locks the mutex and sets.
      */
    void lock()       const  { is_owner = true; mutex.lock(); }
  private:
    const ThreadMutex& mutex;
    mutable bool is_owner;
  };

  /** This class is a wrapper for the pthread_cond_t variables.
   */
  class ThreadCondition {
  public:

    /** Constructor
     */
    ThreadCondition(const ThreadMutex&);

    /** Destructor
     */
    ~ThreadCondition();

    /** timedWait atomically unlocks mutex and  waits, but it also bounds the
     * duration of the wait. If notify has not been signaled within
     * the  amount  of time specified by abstime, the mutex is  re-acquired.
     */
    int timedWait(long sec = 0, long nsec = 0) const;

    /** wait atomically unlocks the mutex and waits for the condition variable
     * to be signaled.
     */
    void wait()              const ;

    /** Restarts one of the threads that  are waiting on this condition variable.
     * If several threads are waiting, exactly one is restarted, but it is not
     * specified which.
     */
    inline void notify()     const {pthread_cond_signal(&cond);}

    /** Restarts all the threads that are waiting on the condition variable.
     */
    inline void notifyAll()  const {pthread_cond_broadcast(&cond);}
  private:
    ThreadCondition(const ThreadCondition&);    //disabled copy contructor
    void operator=(const ThreadCondition&); //disabled assignement operator

    mutable pthread_cond_t cond;
    const ThreadMutex& mutex;
  };


  /** This class is a wrapper for a pthread-threads.
   */
  class Thread {
  public:
    typedef void* (*ThrFunction) (void*);
    /** Constructor
     */
    Thread();

    /** Destructor
     */
    ~Thread();

    /** Creates a new thread.
      *  The new thread applies the function start_routine passing it arg as first
      * argument.  The new thread terminates either explicitly, *
      * \param start_routine : pointer to the start_routine with type void* method(void*)
      * \param args : argument for the start_routine
      * \param attr : pthread attributes(see pthread manual)
      */
    void start(ThrFunction start_routine,	 void *arg, pthread_attr_t* attr = 0 );


    /** Kills the thread.
     */
    void kill();

    /** Blocks until the thread terminates
     */
    int join();

    /** Terminates the execution of the thread
     */
    inline void exit() { pthread_exit(&thread); }

    /** returns the ThreadID
      */
    inline pthread_t getID() { return thread; }

  private:
    pthread_t thread;

  };
}
#endif

