/***************************************************************************
                          l1394_avccamera.h  -  description
                             -------------------
    begin                : Fri Nov 24 2000
    copyright            : (C) 2000-2004 by Michael Repplinger
    email                : repplinger@cs.uni-sb.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef L1394AVCCAMERA_H
#define L1394AVCCAMERA_H

#include "l1394_camera.h"
#include "l1394_avccameraregister.h"


namespace L1394{

/*!\class AvcCamera
  *\ingroup L1394_Device
  *\brief This class implements the interface of a camera, defined in AV/C camera subunit
  * Specification (Version 2.1).
  *
  * The mechanism to create commands is the same, as described in class Camera .
  *
  * The Session class provides also functions to get explicit AvcCameras.
  * So the example in the Camera class documentation can be changed, if you need an AvcCamera.
  *
  * \code
  * Session* session = GetSession();
  * AvcCamera* camera = session->findAvcCamera(); //get explicit a AvcCamera
  * if (camera == NULL)
  * {
  *   cout << "No AvcCamera found" << endl;
  *   exit(0);
  * }
  * if (camera->zoom()->hasFeature())  //if the camera support zoom
  *   camera->zoom()->setValue(camera->zoom()->getMinValue()); //set zoom value to the minimum value.
  * else
  *   cout << " Feature not supported " << endl;
  * \endcode
  * For more information about controling a device see the tutorial.
  *
  *  To start the isochronous transaction, the method init() must be called, to
  *  allocate resources. The camera is set to default parameters, that can be changed
  *  with method setParameter(). For AV/C Cameras you only can change the iso-channel
  * and buffer count.
  *
  *
  * The AvcCamera class is still under develop. The control functions are
  * not tested (I try it with two AV/C cameras and get NOT_IMPLEMENTED response for all
  * features.)
  *
  * @author Michael Repplinger
  */

class FcpNode;
class AvcCamera : public Camera
{
public:
/*! \name AvcCamera constructor
  * These functions create/delete an AvcCamera.
  */
//@{
/*! \fn AvcCamera(FcpNode* parent_node, const int subunit_value)
  * \brief This method creates an AvcCamera.
  * \param FcpNode* : pointer to the parent node.
  * \param subunit_value : The subunit value defines the subunit-type and the subunit-id of the device.
  */
  AvcCamera(FcpNode* parent_node, const int subunit_value);


/*! \fn ~AvcCamera()
  * \brief destructor
  */
  virtual ~AvcCamera();
//@}

/** \name Camera information
  *  These functions return some camera specific information.
  */
//@{
/*! \fn hasVideoFormat(const Videomode videomode)
  * \brief This method tests, if a videomode is supported by a Camera.
  * \return bool : true if the videomode is supported, false if not
  */
  virtual bool hasVideoFormat(const Videomode videomode)   {if (videomode == DEVICE_DV) return true; return false;}


/*! \fn hasFramerate(const Videomode videomode, const Framerate framerate)
  * \brief This method tests, if a framerate, with a videomode is supported by the camera.
  * \param videomode : the videomode for the framerate.
  * \param framerate : the framerate that should be tested.
  * \return bool : true, if a framerate is supported, false if not
  */
  virtual bool hasFramerate(const Videomode videomode,const Framerate framerate);
//@}

/** \name Basic functions
  *  These functions implement the basic functions, defined in abstract class Device.
  */
//@{
  virtual int start();
  virtual int stop();
//@}
/*! \fn setParameter(const int buffercount, const int channel, const Isomode isomode, const Videomode videomode, const Framerate framerate)
  * \brief This method set the parameter of a camera.
  *
  * At this time only the channel parameter has effect.
  * An AV/C camera normally broadcasts the video data on channel 63. They has a
  *  fixed framerate and video format. So the parameters isomode, video_mode and framerate are ignored.
  *
  *  A short sample show an example to init an AvcCamera.
  * \code
  *  Session *session = GetSession();
  *  int framerate = 0;
  *  AvcCamera *camera = session->findAvcCamera();
  *  camera->setParameter() //set the default parameter (channel = 63, buffers = 3)
  *  if (camera->init() == L1394_SUCCESS)
  *    camera->start();
  *  else
  *    cout << "Camera init failed" << endl;
  * \endcode
  *
  * \param isomode : only running mode
  * \param nbuffers : buffers for iso module (for later version. At this time the video
  * module isn't used to get frames from a DV device.
  * \param videomode : DV format
  * \param framerate : CAMERA_FRAMES_3_75, CAMERA_FRAMES_7_5,
  * CAMERA_FRAMES_15, CAMERA_FRAMES_30, CAMERA_FRAMES_60
  * \param channel : iso channel (a value between 1..64)
  * \return int : returns a int value >= 0 if init success else < 0;
  */
  virtual int setParameter(const int buffercount = 4, const int channel = 63, const Isomode isomode = DEVICE_ISO_RUN, const Videomode videomode = DEVICE_DV, const Framerate framerate = DEVICE_FRAMES_DV);


/*! \name AV/C Camera Interface
  * These functions describe the interface of an AV/C camera
  */
//@{
  //! Access 'Brightness' feature register
  virtual const CameraRegister* brightness()   {return (AvcCameraRegister*)avc_camera_register->defaultSet(1,0x49,0xff);}

  //! Access 'Auto-Exposure' feature register
  virtual const CameraRegister* autoExposure() {return (AvcCameraRegister*)avc_camera_register->defaultSet(1,0x40,0xff);}

  //! Access 'Sharpness' feature register
  virtual const CameraRegister* sharpness()     {return (AvcCameraRegister*)avc_camera_register->defaultSet(1,0x56,0xff);}

  //! Access 'White-Balance' feature register
  virtual const CameraRegister* whiteBalance() {return (AvcCameraRegister*)avc_camera_register->defaultSet(1,0x5d,0xff);}

  //! Access 'White-Balance' feature register
  virtual const CameraRegister* whiteBalance(const int) {return (AvcCameraRegister*)avc_camera_register->defaultSet(1,0x5d,0xff);}

  //! Access 'Hue' feature register
  virtual const CameraRegister* hue()           {return (AvcCameraRegister*)avc_camera_register->defaultSet(1,0x5c,0xff);}

  //! Access 'Saturation' feature register
  virtual const CameraRegister* saturation()   {return (AvcCameraRegister*)avc_camera_register->defaultSet(1,0x5b,0xff);}

  //! Access 'Gamma' feature register
  virtual const CameraRegister* gamma()         {return (AvcCameraRegister*)avc_camera_register->defaultSet(1,0x52,0xff);}

  //! Access 'Shutter' feature register
  virtual const CameraRegister* shutter()       {avc_camera_register->setOffset(0x44); return avc_camera_register;}

  //! Access 'Gain' feature register
  virtual const CameraRegister* gain()         {return (AvcCameraRegister*)avc_camera_register->defaultSet(1,0x45,0xff);}

  //! Access 'Iris' feature register
  virtual const CameraRegister* iris()         {return (AvcCameraRegister*)avc_camera_register->defaultSet(1,0x43,0xff);}//check

  //! Access 'Focus' feature register
  virtual const CameraRegister* focus()          {return (AvcCameraRegister*)avc_camera_register->defaultSet(1,0xc1,0xff);}

  //! Access 'Temperature' feature register
  virtual const CameraRegister* temperature()   {avc_camera_register->setOffset(0x2c); return avc_camera_register;}//not c.!!

  //! Access 'Trigger' feature register
  virtual const CameraRegister* trigger()       {avc_camera_register->setOffset(0x30); return avc_camera_register;}//not c.

  //! Access 'Zoom' feature register
  virtual const CameraRegister* zoom()          {return (AvcCameraRegister*)avc_camera_register->defaultSet(1,0x61,0xff);}

  //! Access 'Pan' feature register
  virtual const CameraRegister* pan()           {avc_camera_register->setOffset(0xda); return avc_camera_register;}

  //! Access 'Tilt' feature register
  virtual const CameraRegister* tilt()         {avc_camera_register->setOffset(0xdb); return avc_camera_register;}

  //! Access 'Filter' feature register
  virtual const CameraRegister* filter()       {avc_camera_register->setOffset(0xcb); return avc_camera_register;}
//@}

private:
  //disabled Copy constructor
  AvcCamera (const AvcCamera&);

   //the register object
   AvcCameraRegister* avc_camera_register;

   //Parent node
  FcpNode* avc_parent;

  //is_init
  mutable bool is_init;
  int subunit_value;
  bool frame_mode;

  //iso-parameter
  int isomode, videomode;
  int  buffer,  framerate;
  int  channel;

};
} //namespace L1394



#endif
