/***************************************************************************
                          l1394_avccameraregister.h  -  description
                             -------------------
    begin                : Wed Oct 25 2000
    copyright            : (C) 2000-2004 by Michael Repplinger
    email                : repplinger@cs.uni-sb.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef L1394AVCCAMERAREGISTER_H
#define L1394AVCCAMERAREGISTER_H

#include "l1394_cameraregister.h"
#include "l1394_qarray.h"
#include "l1394_resource.h"
#include "l1394_avcregister.h"


namespace L1394{

/*! \class AvcCameraRegister
  * \ingroup L1394_Register
  * \brief Class AvcCameraRegister implements the interface of class CameraRegister for AvcCameras.
  *
  * As described in class CameraRegister, class AvcCameraRegister defines how to execute a
  *  feature of a Camera based on the AV/C camera subunit Specification (Version 2.1).
  *
  * For example-code see class Camera and AvcCamera.
  *
  * Illegal combinations are ignored.
  *
  *@author Michael Repplinger
  */
class FcpNode;
class AvcCameraRegister : public CameraRegister, public AvcRegister
{
public:

/*! \name CameraRegister constructor
  * These functions implement the CameraRegister interface for AvcCameras
  */
//@{

/*! \fn AvcCameraRegister(const FcpNode* parent_node, const int subunit_value)
  * \brief This method creates a CameraRegister object for AvcCameras.
  * \param FcpNode* : pointer to the parent node.
  * \param subunit_value : The subunit value defines the subunit-type and the subunit-id of the device.
  */
  AvcCameraRegister(const FcpNode* parent_node, const int subunit_value);


/*! \fn ~AvcCameraRegister()
  * \brief destructor
  */
  ~AvcCameraRegister();
//@}

/*! \name Camera_register Interface
  * These functions implement the CameraRegister interface for AvcCameras
  */
//@{
/*! \fn onePush(const bool one_push = true)  const
  * \brief This method activates or deactivates one-push mode of a feature
  * \param one_push : enables or disables one_push
  * \return bool : new status of one-push mode.
  */
   virtual bool onePush(const bool one_push = true)  const;


/*! \fn automatic(const bool on = true)      const
  * \brief This method activates or deactivates the auto-mode of a feature.
  * \param on : enables or disables automatic mode of a feature.
  * \return bool : new status of automatic mode.
  */
   virtual bool automatic(const bool on = true)      const;


/*! \fn setValue(const int value) const
  * \brief This method changes the value of a feature.
  * \param value : new value that should be set.
  * \return int : the new value of the feature.
  */
   virtual int setValue(const int value)            const;


/*! \fn setTriggerMode(const int value) const
  * \brief This method sets the value of the trigger mode.
  * \param value : new value for trigger mode.
  * \return int : value after changing the trigger mode.
  */
   virtual int setTriggerMode(const int value)      const;


/*! \fn setTriggerPolarity(const bool value) const
  * \brief This method enables or disables the trigger polarity
  * \param value : new value that should be set.
  * \return int : the new value of the feature.
  */
   virtual bool setTriggerPolarity(const bool value)const;


/*! \fn getValue() const
  * \brief This method returns the actual value of a feature.
  * \return int : actual value of a feature.
  */
   virtual int getValue()                           const;


/*! \fn getMinValue()  const
  * \brief This method returns the minimum value of a feature
  * \return int : minimum value of a feature
  */
   virtual int getMinValue()                        const;


/*! \fn getMaxValue()  const
  * \brief This method returns the maximum value of a feature.
  * \return int : maximum value of a feature.
  */
   virtual int getMaxValue()                        const;


/*! \fn on(const bool set) const
  * \brief This method enable or disable a feature.
  *
  * After disabling a feature, no write access has affect until enabling
  * the feature again.So this method can be used, to lock (unlock) a
  * feature to a value.
  * \param set : true enable, false disable the feature.
  */
   virtual bool on(const bool =true) const {cout << "not impl." << endl; return false;}


/*! \fn hasFeature() const
  * \brief This method tests if a feature is supported by the camera.
  * \return bool : true if the feature is supported, false if not.
  */
   virtual bool hasFeature()                        const;


/*! \fn hasOnePush() const
  * \brief This method check, if one-push mode is supported by the camera
  * \return bool : true if a feature supported this mode, false if not.
  */
   virtual bool hasOnePush()                        const;


/*! \fn hasReadout() const
  * \brief This method tests for readout mode of a feature.
  * \return bool : true if a feature supports this mode, false if not.
  */
   virtual bool hasReadout()                        const;


/*! \fn hasOnOff()  const
  * \brief This method tests, if on-off mode is support by a feature
  * \return bool : true if a feature support this mode, false if not.
  */
   virtual bool hasOnOff()                          const;


/*! \fn hasAuto()  const
  * \brief This method tests for auto mode of a feature.
  * \return bool : true if mode is supported, false if not.
  */
   virtual bool hasAuto()                            const;


/*! \fn hasManual() const
  * \brief This method tests for manual mode of a feature.
  * \return bool : true if feature has manual support, false if not.
  */
   virtual bool hasManual()                          const;
//@}
/*! \name Some internal functions
  * These functions help to implement the interface.
  */
//@{
   void setOffset(int) {}
//@}
  private:

  AvcCameraRegister(const AvcCameraRegister&);



};
}    //namespace L1394
#endif
