/***************************************************************************
                          l1394_avcregister.h  -  description
                             -------------------
    begin                : Fri Nov 3 2000
    copyright            : (C) 2000-2004 by Michael Repplinger
    email                : repplinger@cs.uni-sb.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef L1394AVCREGISTER_H
#define L1394AVCREGISTER_H

#include "l1394_qarray.h"
#include "l1394_resource.h"
#include "l1394_deviceregister.h"



namespace L1394{
/*! \class AvcRegister
  *  \ingroup L1394_Register
  * \brief This class implements the DeviceRegister interface for devices
  *  based on the AV/C specification.
  *
  * @author Michael Repplinger
  */
class FcpNode;
class AvcRegister : public DeviceRegister
{
public:
/*! \name AvcRegister constructor
  * These functions implement the Register objects for AV/C devices.
  */
//@{

/*! \fn AvcRegister(const FcpNode* parent_node, const int subunit_value)
  * \brief This method creates a Register object for AV/C devices.
  * \param FcpNode* : pointer to the parent node.
  * \param subunit_value : The subunit value defines the subunit-type and the subunit-id of the device.
  */
  AvcRegister(const FcpNode* parent_node, const int subunit_value);


/*! \fn ~AvcRegister()
  *  \brief destructor
  */
  virtual ~AvcRegister();
//@}
/*! \name Register interface for A/VC devices
  * These functiosn implement the minimal DeviceRegister interface for A/VC devices
  */
//@{
/*! \fn send() const
  * \brief This method sends the command and returns the response code.
  *
  * After sending, the internal data Array will be restored and set to size 1.
  * \return QArray : returns the response code.
  */
  int send() const ;

/*! \fn hasFeature() const
  * \brief This method tests, if a command is supported by a node.
  *
  *  For this method the general inquiry command is used.
  * \return bool : returns true, if a node reports an IMPLEMENTED for this feature, false if not.
  */
  bool hasFeature() const ;


/*! \fn hasFeature(const int command) const
  * \brief This method tests, if a submethod of a command is supported by the device.
  *
  * \param command : value for the subcommand (see the specific AV/C
  *  subunit specification for more information)
  * \return bool : return true, if node report an IMPLEMENTED for this feature,  false if not.
  */
  bool hasFeature(const int command) const ;


/*! \fn status() const
  * \brief This method returns the status of subfunction(0xFF normally for the feature itself)
  *
  * \return QArray : QArray with the response code.
  */
  QArray status() const;
//@}


/*! \name Some internal methods
  * These methods help to implement the interface.
  */
//@{
/*! \fn setCtype(const int ctype)
  * \brief This method sets the command type.
  * \param ctype : new command-type.
  * \return AvcRegister* : returns a pointer to this object.
  */
  AvcRegister* setCtype(const int ctype)                { data_array->setByte(0,ctype); return this;}

/*! \fn setSubfunction(const int subfunction)
  * \brief This method sets the subfunction.
  * \param subfunction : new subfunction value.
  * \return AvcRegister* : returns a pointer to this object.
  */
  AvcRegister* setSubfunction(const int subfunction)     {  data_array->setByte(3,subfunction); return this;}

/*! \fn setOpcode(const int opcode)
  * \brief This method sets the opcode.
  * \param opcode : new operation code.
  * \return AvcRegister* : returns a pointer to this object.
  */
  AvcRegister* setOpcode(const int opcode)              { data_array->setByte(2,opcode); return this;}

/*! \fn defaultSet(const int length,const  int opcode,const  int value)
  * \brief This method sets default value for the array. FOR INTERNAL USE ONLY!!
  */
  AvcRegister* defaultSet(const int length,const  int opcode,const  int value);

/*! \fn getData()
  * \brief return the data_array. FOR INTERNAL USE ONLY!!!
  */
  QArray* getData() {return data_array;}
//@}
protected:

  //! This method resets the data_arry
  AvcRegister* reset(const int size);

  //! pointer to the parent node
  const FcpNode* parent_node;

  //! the data_array
  QArray* data_array;

  //! actual op_code
  int opcode;

  //! actual subunit_value
  int subunit_value;

  //! actual command type
  int ctype;

  //! actual subfunction
  int subfunction;
  //! variable to test if the data_array was reset.
  bool restore;

};
}
#endif
