/***************************************************************************
                          l1394_avcvcr.h  -  description
                             -------------------
    begin                : Fri Nov 24 2000
    copyright            : (C) 2000-2004 by Michael Repplinger
    email                : repplinger@cs.uni-sb.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef L1394AVCVCR_H
#define L1394AVCVCR_H
#include "l1394_resource.h"
#include "l1394_vcr.h"
#include "l1394_avcregister.h"


namespace L1394{
/*! \class AvcVcr
  *  \ingroup L1394_Device
  * \brief This class implements the interface of a vcr defined in 'AV/C Tape
  * Recorder/Player Subunit specification'(Version 2.1).
  *
  * The mechanism to create commands is the same as described in class Vcr.
  *
  * This class implements two Vcr interfaces. The first interface is described
  * in class Vcr. The second one is an implementation of the controlling mechanism
  * defined in 'AV/C Tape Recorder Player Subunit Specification'. The difference between
  * the two interfaces is the meaning of functions. For example the play() function
  * of the first interface start a vcr. In the second one the play(int value) method means
  * the Play group, where the value specify how to play, like forward, backward, nextFrame and so on.
  *  The functions from the first interface are mapped to the functions of the second one.<BR>
  *
   * The example from class Vcr can be changed:
  *
  * \code
  * AvcVcr* my_vcr = session->findAvcVcr();
  * if( my_vcr->fastForward()->hasFeature() ) //should normally be true.
  * {
  *   int vcr_response = my_vcr->fastForward()->send(); //normal vcr interface command
  *   int avc_vcr_respone = my_vcr->play(0x75)->send(); //avc_vcr interface to start a vcr.
  *   switch(response) //for all response codes see method send() in class AvcRegister.
  *   {
  *     case ACCEPTED : cout << "Starting the Vcr" << endl; break;
  *     case REJECTED : cout << "Couldn't start the Vcr" << endl; break;
  *     default : cout << "Unhandled case" << endl; break;
  *   }
  * }
  * \endcode
  *
  * If you like to use the AV/C vcr interface, you can cast the vcr, or
  * get explicit an AvcVcr from the Session class.
  *
  * See AV/C tape recorder/player subunit specification and tutorial for more information
  *
  *@author Michael Repplinger
  */
class FcpNode;
class AvcVcr : public Vcr
{
public:
/*! \name AvcVcr constructor
  * These functions creates an AvcVcr.
  */
//@{

/*! \fn AvcVcr(FcpNode* parent_node, const int subunit_value)
  * \brief This method creates a AvcVcr.
  * \param FcpNode* : pointer to the parent node.
  * \param subunit_value : The subunit value defines the subunit-type and the subunit-id of the device.
  */
  AvcVcr(FcpNode* parent_node, const int subunit_value);

/*! \fn ~AvcVcr()
  * \brief destructor
  */
  virtual ~AvcVcr();
//@}

/*! \fn start()
  * \brief This method starts the Vcr and the Iso-Receive.
  * \return int : returns L1394_SUCCESS, if the device start.
  */
  int start() ;



/*! \fn stop()
  * \brief This method stops a Vcr.
  * \return int : returns L1394_SUCCESS if the vcr is stopped
  */
  virtual int stop() ;

/*! \fn setParameter(const int buffercount, const int channel, const Isomode isomode, const Videomode videomode, const Framerate framerate)
  * \brief This method set the parameter of a camera.
  *
  * An AV/C camera normally broadcasts the video data on channel 63. They has a
  *  fixed framerate and video format. So the parameters isomode, video_mode and framerate are ignored.
  *
  *  A short sample show an example to init an AvcCamera.
  * \code
  *  Session *session = Session::getSession();
  *  AvcVcr *vcr = session->findAvcCVcr();
  *  vcr->setParameter() //set the default parameter (channel = 63, buffers = 4)
  *  if (camera->init() == L1394_SUCCESS)
  *    camera->start();
  *  else
  *    cout << "Camera init failed" << endl;
  * \endcode
  *
  * \param isomode : only running mode
  * \param nbuffers : buffers for iso module (for later version. At this time the video
  * module isn't used to get frames from a DV device.
  * \param videomode : DV format
  * \param framerate : CAMERA_FRAMES_3_75, CAMERA_FRAMES_7_5,
  * CAMERA_FRAMES_15, CAMERA_FRAMES_30, CAMERA_FRAMES_60
  * \param channel : iso channel (a value between 1..64)
  * \return int : returns a int value >= 0 if init success else < 0;
  */
  virtual int setParameter(const int buffercount = 4, const int channel = 63, const Isomode isomode = DEVICE_ISO_RUN, const Videomode videomode = DEVICE_DV, const Framerate framerate = DEVICE_FRAMES_DV);


/*! \fn hasVideoFormat(const Videomode videomode)
  * \brief This method tests, if a videomode is supported by a Camera.
  * \return bool : true if the videomode is supported, false if not
  */
  virtual bool hasVideoFormat(const Videomode videomode)   {if (videomode == DEVICE_DV) return true; return false;}


/*! \fn hasFramerate(const Videomode videomode, const Framerate framerate)
  * \brief This method tests, if a framerate, with a videomode is supported by the camera.
  * \param videomode : the videomode for the framerate.
  * \param framerate : the framerate that should be tested.
  * \return bool : true, if a framerate is supported, false if not
  */
  virtual bool hasFramerate(const Videomode videomode,const Framerate framerate);


/*! \name The standard functions of the vcr interface
  * This functions implements the interface of class Vcr
  */
//@{

/*! \fn play()
  * \brief With this method you can access the play feature.
  * \return DeviceRegister* : pointer to the DeviceRegister.
  */
  virtual int play()                  {return play(0x75)->send();}


/*! \fn pause()
  * \brief With this method you can access the pause feature.
  * \return DeviceRegister* : pointer to the DeviceRegister.
  */
   virtual int pause()                 {return play(0x7d)->send();}

/*! \fn timeCode getTimeCode()
  * \brief With this method you can get the actual timecode.
  * \return Timecode : the actual timecode.
  */
  virtual Timecode getTimeCode() ;

/*! \fn transportState()
  * \brief With this method you can get the actual status of a vcr.
  * \return DeviceRegister* : pointer to the DeviceRegister.
  */
   virtual const DeviceRegister* transportState()  ;

/*! \fn search(const Timecode)
  * \brief This method stop the tape at a specific position.
  * \return DeviceRegister* : pointer to the DeviceRegister.
  */
  virtual const DeviceRegister* search(const Timecode)  {return device_register;}
  //all following ok

/*! \fn recordingDate(const Timecode time_begin)
  * \brief With this method you can access the recording date feature.
  * \return DeviceRegister* : pointer to the DeviceRegister.
  */
  virtual const DeviceRegister* recordingDate(const Timecode) ;

/*! \fn forward()
  * \brief With this method you can access the forward feature.
  * \return DeviceRegister* : pointer to the DeviceRegister.
  */
  virtual const DeviceRegister* forward()               {return play(0x39);}

/*! \fn reverse()
  * \brief With this method you can access the reverse feature.
  * \return DeviceRegister* : pointer to the DeviceRegister.
  */
  virtual const DeviceRegister* reverse()               {return play(0x48);}

/*! \fn fastForward()
  * \brief With this method you can access the fastForward feature.
  * \return DeviceRegister* : pointer to the DeviceRegister.
  */
  virtual const DeviceRegister* fastForward()           {return wind(0x75);}

/*! \fn slowForward()
  * \brief With this method you can access the slowForward feature.
  * \return DeviceRegister* : pointer to the DeviceRegister.
  */
  virtual const DeviceRegister* rewind()                {return wind(0x65);}

/*! \fn record()
  * \brief With this method you can access the record feature.
  * \return DeviceRegister* : pointer to the DeviceRegister.
  */
  virtual const DeviceRegister* record()                {return record(0x75);}

/*! \fn recordingSpeed(const int speed)
  * \brief With this method you can access the recordingSpeed feature.
  * \param speed : the speed value.
  * \return DeviceRegister* : pointer to the DeviceRegister.
  */
  virtual const DeviceRegister* recordingSpeed(const int speed);

/*! \fn nextFrame()
  * \brief With this method you can access the nextFrame feature.
  *
  * If you execute the command, the vcr increase one frame, from actual
  * position.
  * \return DeviceRegister* : pointer to the DeviceRegister.
  */
  virtual const DeviceRegister* nextFrame()             {return play(0x30);}

/*! \fn previousFrame()
  * \brief With this method you can access the nextFrame feature.
  *
  * If you execute the command, the vcr decrease one frame, from actual
  * position.
  * \return DeviceRegister* : pointer to the DeviceRegister.
  */
  virtual const DeviceRegister* previousFrame()         {return play(0x40);}

/*! \fn slowForward()
  * \brief With this method you can access the slowForward feature.
  * \return DeviceRegister* : pointer to the DeviceRegister.
  */
  virtual const DeviceRegister* slowForward()           {return play(0x3f);}

/*! \fn slowReverse()
  * \brief With this method you can access the slowReverse feature.
  * \return DeviceRegister* : pointer to the DeviceRegister.
  */
  virtual const DeviceRegister* slowReverse()           {return play(0x47);}
//@}
//end of the standard interface of a vcr.

/*! \name The extended functions for an AV/C vcr
  * This functions describe the interface of a vcr defined in AV/C Tape Recorder/
  * Player Subunit Specification.
  */
//@{
/*! \fn analogAudioOutputMode(const int mode)
  * \brief With this method you can access the analog audio output mode
  * \param mode : int value that describe the mode (for valid values see
  * AV/C Tape Recorder Player Subunit Specification.
  * \return AvcRegister* : pointer to the AvcRegister.
  */
  //ok
  const AvcRegister* analogAudioOutputMode(const int);


/*! \fn backward(int measurement_unit, int count)
  * \brief access backward feature and return a pointer to the AvcRegister
  * \param measurement_unit : define which unit should be used (for valid values see
  * AV/C Tape Recorder Player Subunit Specification.
  * \param count : int value for the count of backward steps (valid value between 0x0..0xff)
  * \return AvcRegister* : pointer to the AvcRegister.
  */
  //ok
  const AvcRegister* backward(const int measurement_unit, const int count);


/*! \fn forward(const int measurement_unit,const int count)
  * \brief access backward feature and return a pointer to the AvcRegister
  * \param measurement_unit : define which unit should be used (for valid values see
  * AV/C Tape Recorder Player Subunit Specification.
  * \param count : int value for the count of forward steps (valid value between 0x0..0xff)
  * \return AvcRegister* : pointer to the AvcRegister.
  */
  //ok
  const AvcRegister* forward(const int measurement_unit,const int count)  ;


/*! \fn editMode(const int mode)
  * \brief With this method you can access the analog audio output mode
  * \param mode : int value that describe the mode (for valid values see
  * AV/C Tape Recorder Player Subunit Specification.
  * \return AvcRegister* : pointer to the AvcRegister.
  */
  //ok
  const  AvcRegister* editMode(const int command)   {return device_register->defaultSet(1, 0x40, command);}


/*! \fn inputSignalMode(const int mode)
  * \brief With this method you can access the input signal mode
  * \param mode : int value that describe the mode (for valid values see
  * AV/C Tape Recorder Player Subunit Specification.
  * \return AvcRegister* : pointer to the AvcRegister.
  */
  //ok
  const AvcRegister* inputSignalMode(const int command)  {return device_register->defaultSet(1,0x79,command);}


/*! \fn loadMedium(const int command)
  * \brief With this method you can access the load medium feature.
  * \param command : int value that describe the command (for valid values see
  * AV/C Tape Recorder Player Subunit Specification.
  *
  * For calling hasFeature command = 0xFF
  * \return AvcRegister* : pointer to the AvcRegister.
  */
  //ok
  const AvcRegister* loadMedium(const int command)  {return device_register->defaultSet(1,0xc1, command);}


/*! \fn marker(const int set)
  * \brief With this method you can access the marker mode
  * \param set : int value 1 for set marker, 0 for unset marker.
  * \return AvcRegister* : pointer to the AvcRegister.
  */
  //ok
  const AvcRegister* marker(const int set)   {return device_register->defaultSet(1,0xca, 1); device_register->getData()->getQuadlet(1).setBit(7,set);return device_register;}


/*! \fn mediumInfo()
  * \brief With this method you can access the analog audio output mode.
  * \return Quadlet : Quadlet with the medium info.
  */
  //ok
  Quadlet mediumInfo()  ;


/*! \fn openMic(const int mode)
  * \brief With this method you can access the open Mic feature
  * \param mode : int value that describe the mode (for valid values see
  * AV/C Tape Recorder Player Subunit Specification.
  * \return AvcRegister* : pointer to the AvcRegister.
  */
  //ok
  const AvcRegister* openMic(const int command)   {return device_register->defaultSet(1,0x60,command);}


/*! \fn outputSignalMode(const int command)
  * \brief With this method you can access the output signal mode
  * \param command : int value that describe the command (for valid values see
  * AV/C Tape Recorder Player Subunit Specification.
  * \return AvcRegister* : pointer to the AvcRegister.
  */
  //ok
  const AvcRegister* outputSignalMode(const int command)   {return device_register->defaultSet(1,0x78, command);}


/*! \fn record(const int command)
  * \brief With this method you can access the record feature.
  * \param mode : int value that describe the mode (for valid values see
  * AV/C Tape Recorder Player Subunit Specification.
  * \return AvcRegister* : pointer to the AvcRegister.
  */
  //ok
  const AvcRegister* record(const int command)     {return device_register->defaultSet(1,0xc2, command);}


/*! \fn timecode(const int command,const Timecode timecode)
  * \brief With this method you can access the timecode feature.
  * \param command : int value that describe the mode (for valid values see
  * AV/C Tape Recorder Player Subunit Specification.
  * \param timecode : the new timecode. In case of ctype STATUS, all variable of
  * timecode has  value 0xff
  * \return AvcRegister* : pointer to the AvcRegister.
  */
  //ok
   const AvcRegister* timecode(const int command,const  Timecode timecode)         ;


/*! \fn writeMic()
  * \brief With this method you can access the write mic feature.
  * \return AvcRegister* : pointer to the AvcRegister.
  */
   const AvcRegister* writeMic()         ; //not implemented


/*! \fn wind(const int command)
  * \brief With this method you can access the wind feature.
  * \param command : int value that describe the subfunction(for valid values see
  * AV/C Tape Recorder Player Subunit Specification.
  * \return AvcRegister* : pointer to the AvcRegister.
  */
  //ok
   const AvcRegister* wind(const int command)            ;


/*! \fn play(const int command)
  * \brief With this method you can access the play feature.
  * \param command : int value that describe the subfunction(for valid values see
  * AV/C Tape Recorder Player Subunit Specification.
  * \return AvcRegister* : pointer to the AvcRegister.
  */
  //ok
  const AvcRegister* play(const int value)              {return device_register->defaultSet(1,0xc3,value);}


/*! \fn search(const int command,const  Timecode timecode)
  * \brief With this method you can access the analog audio output mode
  * \param command : int value that describe the submethod (for valid values see
  * AV/C Tape Recorder Player Subunit Specification.
  * \param timecode : the new timecode. In case of ctype STATUS, all variable of
  * timecode has  value 0xff
  * \return AvcRegister* : pointer to the AvcRegister.
  */
  const AvcRegister* search(const int,const  Timecode)            ;



/*! \fn preset(const int reset,const int id,const u_int32_t value)
  * \brief With this method you can access the analog audio output mode
  *   for valid values see AV/C Tape Recorder Player Subunit Specification.
  * \return AvcRegister* : pointer to the AvcRegister.
  */
  const AvcRegister* preset(const int reset,const  int id,const  u_int32_t parameter_value)  ;
//@}


private:
  AvcRegister* device_register;
  int  subunit_value;
  bool frame_mode;
  const  FcpNode* avc_parent;
};
}
#endif
