/***************************************************************************
                          l1394_camera.h  -  description
                             -------------------
    begin                : Fri Nov 24 2000
    copyright            : (C) 2000-2004 by Michael Repplinger
    email                : repplinger@cs.uni-sb.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CAMERA_H
#define CAMERA_H
#include "l1394_cameraregister.h"
#include "l1394_device.h"
namespace L1394
{
#define RED 0
#define BLUE 1

/*! \class Camera
  *  \ingroup L1394_Device
  * \brief   This class defines the interface for cameras.
  *
  * This class is base class for all cameras and defines functions for the most common
  *  features. <BR>
  *  To access a feature, you must call the corresponding function. Normally this
  *  method returns a pointer to a so called L1394 Register object. The Register object
  *  provides functions to define how to execute the feature. Class CameraRegister defines
  *  the functions of the L1394 Register objects for cameras.
  *
  * Here is a little example to show the idea.<BR>
  *
  * \code
  * Session*  session  = GetSession();
  * Camera*  camera  = session->findCamera();
  * if (camera == NULL)
  * {
  *   cout << "No camera found" << endl;
  *   exit(0);
  * }
  * if (camera->zoom()->hasFeature())  //if the camera support zoom
  *   camera->zoom()->setValue(camera->zoom()->getMinValue()); //set zoom value to the minimum value.
  * else
  *   cout << " Feature not supported " << endl;
  * \endcode
  * For more information see the tutorial.
  *
  *  @author Michael Repplinger
  */


class Camera : public Device
{
public:
/*! \name Camera constructor
  * These functions creates the Camera objects
  */
//@{
  Camera (Node* parent_node);
/*! \fn ~Camera
  * \brief destructor
  */
  virtual ~Camera(){};

//@}
  virtual DeviceType getDeviceType() const { return _Camera;}
/*! \name Camera Interface
  * This functions describe the interface of any camera. There are combinations
  *  to create an illegal command. These commands are ignored.
  */
//@{
  //! Access 'Brightness' feature register
  virtual const CameraRegister* brightness()          =0;

  //! Access 'Auto-Exposure' feature register
  virtual const CameraRegister* autoExposure()       =0;

  //! Access 'Sharpness' feature register
  virtual const CameraRegister* sharpness()            =0;

  //! Access 'White-Balance' feature register (change both RED and BLUE)
  virtual const CameraRegister* whiteBalance()        =0;

  //! Access 'White-Balance' feature register 1 = RED ,BLUE = 0
  virtual const CameraRegister* whiteBalance(int)     =0;

  //! Access 'Hue' feature register
  virtual const CameraRegister* hue()                 =0;

  //! Access 'Saturation' feature register
  virtual const CameraRegister* saturation()         =0;

  //! Access 'Gamma' feature register
  virtual const CameraRegister* gamma()                =0;

  //! Access 'Shutter' feature register
  virtual const CameraRegister* shutter()             =0;

  //! Access 'Gain' feature register
  virtual const CameraRegister* gain()               =0;

  //! Access 'Iris' feature register
  virtual const CameraRegister* iris()               =0;

  //! Access 'Focus' feature register
  virtual const CameraRegister* focus()               =0;

  //! Access 'Temperature' feature register
  virtual const CameraRegister* temperature()         =0;

  //! Access 'Trigger' feature register
  virtual const CameraRegister* trigger()             =0;

  //! Access 'Zoom' feature register
  virtual const CameraRegister* zoom()               =0;

  //! Access 'Pan' feature register
  virtual const CameraRegister* pan()                 =0;

  //! Access 'Tilt' feature register
  virtual const CameraRegister* tilt()               =0;

  //! Access 'Filter' feature register
  virtual const CameraRegister* filter()             =0;
//@}

};
}
#endif
