/***************************************************************************
                          l1394_cameraregister.h  -  description
                             -------------------
    begin                : Fri Nov 24 2000
    copyright            : (C) 2000-2004 by Michael Repplinger
    email                : repplinger@cs.uni-sb.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CAMERAREGISTER_H
#define CAMERAREGISTER_H
//ok
namespace L1394
{

/**\class CameraRegister
  *\ingroup L1394_Register
  *\brief This class defines the interface of an L1394 Register for Cameras.
  *
  *  If you access a feature of a device a so called L1394 Register object is returned.
  *  The L1394 Register object provides functions to define how to execute the feature.
  *
  * The functions of this class define, how to execute commands of a Camera.<BR>
  *  If you store a pointer to a L1394 Register object remember, that the functions of
  *  this class are only valid for the last feature called by the Device.
  *
  * Functions changing the status of the camera, like setValue(int) return the new value.
  * So you can test, if the new value is set correct.
  *
  * Illegal combinations are ignored.
  *
  *  For an example see class Camera.
  *
  *@author Michael Repplinger
  */

class CameraRegister {
public:
/*! \name CameraRegister constructor
  * These functions creates the Register objects for Camera devices.
  */
//@{

/*! \fn ~CameraRegister()
  * \brief destructor
  */
  virtual  ~CameraRegister(){};
//@}
/*! \name Camera_register Interface
  * These functions defines how to execute a feature of class Camera.
  */
//@{
/*! \fn onePush(const bool one_push = true)  const
  * \brief This method activates or deactivates one-push mode of a feature
  * \param one_push : enable or disable one_push
  * \return bool : new status of one-push mode.
  */
  virtual bool onePush(const bool one_push = true)  const  =0;


/*! \fn automatic(const bool on = true)      const
  * \brief This method activates or deactivates the automode of a feature.
  * \param on : enable or disable automatic mode for a feature.
  * \return bool : new status of automatic mode.
  */
  virtual bool automatic(const bool on = true)      const  =0;


/*! \fn setTriggerMode(const int value) const
  * \brief This method sets the trigger mode.
  *
  * This method works only combined with the trigger feature.
  * \param value : new value for trigger mode.
  * \return int : value after changing the trigger mode.
  */
  virtual int setTriggerMode(const int value)         const  =0;


/*! \fn setTriggerPolarity(const bool value) const
  * \brief This method sets trigger polarity to value (False = low active)
  *
  * This method works only combined with the trigger feature.
  * \param value : new value for trigger mode.
  * \return bool : status after changing trigger mode
  */
  virtual bool setTriggerPolarity(const bool value) const  =0;


/*! \fn setValue(const int value) const
  * \brief This method change the value of a feature.
  * \param value : new value for the feature
  * \return int :  value after changing new value.
  */
  virtual int setValue(const int value)              const  =0;


/*! \fn getValue() const
  * \brief This method returns the current value of a feature.
  * \return int : value of a feature.
  */
  virtual int getValue()                            const  =0;


/*! \fn getMinValue()  const
  * \brief This method returns the minimal value of a feature.
  * \return int : minimal value of a feature
  */
  virtual int getMinValue()                          const  =0;


/*! \fn getMaxValue()  const
  * \brief This method returns the maximal value of a feature.
  * \return int : maximal value of a feature
  */
  virtual int getMaxValue()                         const  =0;


/*! \fn on(const bool set) const
  * \brief This method enables or disables a feature.
  *
  * After disabling a feature, no write access has affect until enabling
  * the feature again. So this method can be used, to lock (unlock) a
  * feature to a value.
  * \param set : true enable, false disable the feature.
  */
  virtual bool on(const bool =true)                 const  =0;


/*! \fn hasFeature() const
  * \brief This method tests if a feature is supported by a camera.
  * \return bool : true if feature is supported, false if not.
  */
  virtual bool hasFeature()                          const  =0;


/*! \fn hasOnePush() const
  * \brief This method tests, if one-push mode is supported by a camera.
  * \return bool : true if a feature supported this mode, false if not.
  */
  virtual bool hasOnePush()                          const  =0;


/*! \fn hasReadout() const
  * \brief This method tests for readout mode of a feature.
  * \return bool : true if a feature support this mode, false if not.
  */
  virtual bool hasReadout()                          const  =0;


/*! \fn hasOnOff()  const
  * \brief This method tests, if on-off mode is support by a feature.
  * \return bool : true if a feature support this mode, false if not.
  */
  virtual bool hasOnOff()                            const   =0;


/*! \fn hasAuto()  const
  * \brief This method tests for auto mode of a feature.
  * \return bool : true if mode is supported, false if not.
  */
  virtual bool hasAuto()                            const  =0;


/*! \fn hasManual() const
  * \brief This method tests for manual mode of a feature.
  * \return bool : true if feature has manual support, false if not.
  */
  virtual bool hasManual()                          const  =0;

//@}
};
}
#endif
