/***************************************************************************
                          l1394card.h  -  description
                             -------------------
    begin                : Wed Jul 5 2000
    copyright            : (C) 2000-2004 by Michael Repplinger
    email                : repplix@studcs.uni-sb.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef L1394CARD_H
#define L1394CARD_H

#include "l1394_node.h"
#include "l1394_nodecontainer.h"
#include "l1394_nodefactory.h"
#include "l1394_csrdirectory.h"
#include "l1394_bustopology.h"
#include "l1394_message.h"
#include "l1394_transaction.h"
#include "Thread.hpp"

#include <list>


namespace L1394{
/*! \class Card
  * \ingroup L1394_Node
  * \brief This class represents a FireWire card.
  *
  * A Card manages up to 64 (pointer of) Nodes in an array.
  * The Nodes are stored in a global internal::NodeContainer except Nodes with a physical layer only,
  * so called PhyNodes(because there is no way to identify them). PhyNodes are
  * deleted after every bus reset (and recreate if they are always
  * connected to the FireWire bus.)
  *
  * To guarantee that every node connected to the bus is represented by one
  * L1394 node the Card searches the global internal::NodeContainer for an
  * existing object. So its possible to connect one device to multiple cards and work always with the same object.
  *
  * Every card object provides a Transaction object (wrapper class for libraw1394)
  * that can be used to send asynchone transactions.
  *
  * This class provides also functions to find and get Nodes, but they search on
  * the current card only. The Card returns objects from type L1394::Node, so you need to cast
  * the L1394 nodes to the correct type if you need. <BR>
  * To find and get nodes you should use the functions from class Session.
  *
  * @author Michael Repplinger
  */


class Card : public Node
{
public:
/*! \name Camera constructor
  * These functions creates the Card objects.
  */
//@{
/*! \fn Card(const u_int32_t card_number, const u_int32_t node_number, raw1394handle_t default_handle)
  * \brief constructor
  * \param card_number : specify the card number.
  * \param node_number : specify the node_number by creating this object
  * \param default_handle: pointer to the default handle for this node
  */
  Card(const u_int32_t card_id, const u_int32_t node_id, raw1394handle_t default_handle);


/*! \fn ~Card()
 *  \brief destroy an 1394 Card object
 */
  virtual  ~Card();
//@}


/** \name Access the connected nodes
  *  These functions access the connected nodes on a FireWire card. All functions
  *  search only on this card.
  */
//@{
/*! \fn getNode(const u_int64_t guid) const
 *  \brief This method searches a Node with a certain guid.
 *
 *  This method searches only on this card.
 *  \param guid : the guid of the node
 *  \return Node* : pointer to the node, NULL if the node doesn't exist on this card.
 */
  Node *getNode(const u_int64_t guid)          const;


/*! \fn getAllNodes(const int node_type) const
 *  \brief This method searches all nodes with a certain type and
 *   returns a STL-list.
 *
 *  This method searches only on this card.
 *  \param node_type : integer for type of the nodes.
 *  \return list<Node*> : stl_list with Nodes from type node_type
 */
  list<Node*> getAllNodes(const int node_type) const;


/*! \fn findNode(const int node_type) const
 *  \brief This method searches the first node with a certain node type.
 *
 *  This method searches only on this card.
 *  \param node_type : integer value with the node type (defined in class Node)
 *  \return Node* : pointer to the first node with this node_type, NULL if no node
 *    exist on this card.
 */
  Node *findNode(const int node_type)          const;


/*! \fn getNodeArray() const
 *  \brief This method returns the array of all nodes on the card.
 *
 *  The node_array stores all Nodes connected to this card.
 *  The array has length 64 and is filled up with NULL-pointer, if it contains
 *  less than 64 nodes. The content of this array can change after bus reset.
 *  \return Node** : pointer to the Node array with size 64.
 */
  Node **getNodeArray()                        const {return node_array;}


/*! \fn getExplicitNode(const int i) const
 *  \brief This method returns the i'th node on the card.
 *  \param i : for the i'th node.
 *  \return Node* : pointer to the Node, NULL if no node with this id exist.
 */
  Node* getExplicitNode(const int i)           const {return node_array[i];}


/*! \fn nodeExist(const u_int64_t guid) const
 *  \brief This method tests if a node with a certain guid is connected to this card.
 *
 *  This method searches only on this card.
 *  \param guid : the guid of the node
 *  \return bool : value true for node exist, else false.
 */
  bool nodeExist(const u_int64_t guid)         const;


/*! \fn getBusMaster() const
 *  \brief This method returns the actual Bus-Master node. The busmaster
 *  can change after bus reset.
 *  \return Node* : pointer to the actual busmaster.
 */
  Node* getBusMaster() const {if (node_count >=1 ) return node_array[node_count-1]; return NULL;}
//@}


/** \name Card specific information
  *  These functions return some card specific information.
  */
//@{
/*! \fn getNodeCount() const
 *  \brief This method returns the number of connected nodes.
 *  \return int : value with node count (between 1..64)
 */
  u_int32_t getNodeCount()                const {return node_count;}


/*! \fn getCardID() const
  * \brief This method returns the card-id
  */
  u_int32_t getCardID()                   const {return card_id;}


/*! \fn getDefaultHandle() const
  * \brief This method returns the current default-handler (obsolete)
  */
  raw1394handle_t getDefaultHandle()      const {return default_handle;}


/*! \fn getBusTopology() const
 *  \brief This method returns the actual BusTopology.
 *
 *  An object of this class stores the actual Topo Map as QArray and as tree.
 *  The bustopology  changes after every bus reset.
 *  \return internal::BusTopology* : pointer to the actual BusTopology
 */
  internal::BusTopology* getBusTopology() const { return bustopology;}


/*! \fn printNodeList() const
 *  \brief prints the device list of a card to standard output
 */
  void printNodeList()                    const;


/*! \fn getTopoMap() const
  * \brief This method returns the actual topology map
  * \return QArray* : pointer to the topology map
*/
  QArray* getTopoMap()                    const {return topo_map;}

/** resets the bus
  */
  int reset() const;
//@}


/** \name Transaction object
  *  These methods access the Transaction object of a card.
  */
//@{
/*! \fn requestTransaction() const
  * \brief This method returns the Transaction object of a Card.
  *
  * This method is for internal use. You should use the Node memberfunctions
  * read, write and lock.
  * Remark: The Transaction object must be released before any Node can
  * request it again. So if you must use this method you MUST call
  * the method releaseTransaction() to release the Transaction object.
  */
  const internal::Transaction* requestTransaction() const ;


/*! \fn releaseTransaction() const
  * \brief This method releases the Transaction object
  */
  void releaseTransaction()               const;
//@}

/*! \fn makeNodeList(raw1394handle_t)
 *  \brief This method creates the internal node list.
 */
  virtual void makeNodeList(raw1394handle_t);

/*
  returns -1, if no channel available
*/
  int requestIsoChannel() const;
  int requestIsoChannel(const u_int32_t) const;
  void releaseIsoChannel(const u_int32_t) const;
protected:

/*! \fn readAGuid(const int node);
  * \brief This method reads the GUID of a node connected to this card direct from CSR.
  * \return u_int64_t : Return the GUID.
  */
  u_int64_t readAGuid(const int node);


/*! \fn readTopoMap()
  * \brief This method reads the Topologie map
  */
  void readTopoMap();

private:
  /* Disables Copy constructor */
  Card(const Card&);

  // releases all;
  //void  makeClean();

  // pointer to the NodeContainer object
  internal::NodeContainer* node_container;

  // pointer to the csr-object (defined in l1394csr.h)
  internal::CsrRom* rom_info;

  // pointer to the bustopology
  internal::BusTopology* bustopology;

  // The transaction object
  internal::Transaction* transaction;

  // pointer to actual topology map
  QArray* topo_map;

  // counter for nodes and bus reset
  u_int32_t node_count, reset_count, card_id;

  // the node array
  Node **node_array;

  // mutex
  ThreadMutex mutex;

  // The default_handle
  raw1394handle_t default_handle;


  mutable Quadlet low_iso_channel;
   mutable Quadlet high_iso_channel;
};
}    //namespace L1394
#endif
