/***************************************************************************
                          l1394csrdirectory.h  -  description
                             -------------------
    begin                : Fri Oct 27 2000
    copyright            : (C) 2000-2004 by Michael Repplinger
    email                : repplinger@cs.uni-sb.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef L1394CSRDIRECTORY_H
#define L1394CSRDIRECTORY_H

#include "l1394_qarray.h"
//document_status : final
//debug_status :

namespace L1394{
namespace internal{
class CsrDirectory;
/*! \class DNode
  * \ingroup Internal
  * \brief A DNode represents a node of the CsrDirectory.
  *
  * There a four node types that store information the CsrRom:
  *  The type is stored as integer value.
  * DNode-type immediate : <BR>
  * This DNode stores the information of an immediate entry in CsrRom.
  *
  * DNode-type offset : <BR>
  * This DNode stores the information of an offset entry in CsrRom.
  *
  * DNode-type leaf : <BR>
  * This DNode stores the information of an leaf entry in CsrRom.
  *
  * DNode type directory : <BR>
  * This DNode stores a pointer to a CsrDirectory in CsrRom.
  *
  *  @author Michael Repplinger
  */


class DNode {

static const char *nodeType[];
static const char *keyValue[];

public:

/*! \fn DNode()
  * \brief Constructor
  */
  DNode();

/*! \fn   DNode(int type, QArray *node_info)
  * \brief Constructor
  * \param type : integer value for the DNode type.
  * \param node_info : pointer with the node information
  */
  DNode(int type, QArray *node_info);


/*! \fn  DNode(int type, CsrDirectory* subdirectory)
  * \brief Constructor
  * \param type : integer value for DNode type
  * \param subdirectory : pointer to the subdirectory.
  */
  DNode(int type, CsrDirectory *subdir);


/*! \fn  ~DNode()
  * \brief Destructor
  */
  ~DNode();

/*! \fn print()
  * \brief This method prints the content of a DNode to standard output.
  */
  void print();


/*! \fn getType() const
  * \brief This method returns the type of a DNode.
  * \return int : the DNode type.
  */
  int getType() const {return type;}


/*! \fn getInfo() const
  * \brief This method returns a pointer to the information stored in the DNode.
  * \return QArray* : pointer to the information, NULL if it stores a subdirectory.
  */
  QArray* getInfo() const {return leafInfo;}


/*! \fn getSubDir() const
  * \brief This method returns the subdirectory of a DNode.
  * \return CsrDirectoy* : a pointer to the subdirectory, NULL if the DNode has no
  * subdirectory.
  */
  CsrDirectory* getSubDir() const {return subDir;}

private:

  int type;
  CsrDirectory *subDir;
  QArray *leafInfo;
};


/*! \class CsrDirectory
  * \ingroup Internal
  * \brief An object of this class represent the CSR directory of a CSR rom.
  *
  * Every CSR rom has at least one CSR directory to store some information.
  * A CsrDirectory object represent a CSR directory and stores the information
  * in DNodes.
  *
  * For more information about CSR rom and the data structures see IEEE1212
  * specification.
  *
  *  @author Michael Repplinger
  */


class CsrDirectory {


public:

/*! \fn CsrDirectory(int i,CsrDirectory* )
  * \brief Constructor for a directory of length i and parent directory p.
  *  For a root directory the parent is NULL
  */
  CsrDirectory(int i,CsrDirectory*);

/*! \fn ~CsrDirectory()
  * \brief Destructor
  */
  ~CsrDirectory();


/*! \fn print()
  * \brief This method prints the content of a directory to standard output.
 */
  void print() { for (int i=0; i<count; i++) root_directory[i]->print();}


/*! \fn getParent() const
  * \brief This method returns the parent of a directory.
  * \return CsrDirectory* : returns a pointer to the parent directory, NULL if it's a root directory
  */
  CsrDirectory* getParent() const { return parent;}


/*! \fn findSubDir(int key_id, CsrDirectory* start_directory)
  *  \brief This method searches  a directory with a specific key_id.
  *
  *  The key_id are the last 6 bit of the key
  * \param key_id : integer value for the key_id
  * \param start_directory : pointer to the start directory
  * \return CsrDirectory : pointer to the CsrDirectory with key_id,
  * NULL if no such directory exist.
  */
  CsrDirectory *findSubDir(int key_id, CsrDirectory *startDirectory);


/*! \fn getDNode(int i)
  * \brief This method returns the i'th DNode of the directory
  * \param i : integer value for the i'th DNode.
  * \return DNode* : pointer to the i'th entry of the directory
  */
  DNode *getDNode(int i) {if (i < count) return root_directory[i]; else return NULL;}


/*! \fn attach(DNode* entry)
  * \brief This method adds an DNode to the root_directory
  * \param entry : pointer to the new DNode.
  */
  void attach(DNode *d) {if (count < max_count) { root_directory[count] = d; count++;}}

/*! \fn findNode(int key_id, CsrDirectory *start_directory)
  * \brief This method searches a node with key_id and returns a pointer to it.
  *
  * The search starts at start_directory.
  * \param key_id : integer value with the key_id
  * \param start_directory : pointer to the start directory.
  * \return DNode* : pointer to the directory node, NULL if no such DNode exist.
  */
  DNode *findNode(int key_id, CsrDirectory *startDirectory);


/*! \fn findNode(int key_id, CsrDirectory *startDirectory)
  * \brief This method searches a node with a specific key_id.
  *
  * The search start in this directory.
  * \param key_id : integer value with the key-id
  * \return DNode* : pointer to the DNode with key_id, NULL if no such DNode exist.
  */
  DNode *findNode(int key_id) {return findNode(key_id, this);}


/*! \fn getLength() const
  * \brief returns the size of the directory.
  * \return the size of the directory
  */
   int getLength() const {return count;}

private:

  //disable copy constructor
  CsrDirectory(const CsrDirectory&);

  DNode **root_directory;
  CsrDirectory *parent;
  int max_count, count;
};
}
}
#endif
