/***************************************************************************
                          l1394_dcccamera.h  -  description
                             -------------------
    begin                : Fri Nov 24 2000
    copyright            : (C) 2000-2004 by Michael Repplinger
    email                : repplinger@cs.uni-sb.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef DCCCAMERA_H
#define DCCCAMERA_H
#include "l1394_camera.h"
#include "l1394_node.h"
#include "l1394_dcciso.h"
#include "l1394_dccregister.h"

//ok
namespace L1394{

/*! \class DccCamera
  * \ingroup L1394_Node
  * \ingroup L1394_Device
  * \brief   This class implements the interface of a camera defined in the 1394-based Digital Camera
  * Specification. (Version 1.20)
  *
  * The mechanism to create commands is the same, as described in class Camera .
  *
  * The Session class provides also functions to get explicit DccCameras.
  * So the little example in the Camera class documentation can be changed, if you need a DccCamera.
  *
  * \code
  * Session* session = GetSession();
  * DccCamera* camera = session->findDccCamera(); //get explicit a DccCamera
  * if (camera == NULL)
  * {
  *   cout << "No DccCamera found" << endl;
  *   exit(0);
  * }
  * if (camera->zoom()->hasFeature())  //if the camera support zoom
  *   camera->zoom()->setValue(camera->zoom()->getMinValue()); //set zoom value to the minimum value.
  * else
  *   cout << " Feature not supported " << endl;
  * \endcode
  * For more information see the tutorial.
  *
  *@author Michael Repplinger
  */
class DccCamera : public Node, public Camera
{
public:
/*! \name DccCamera constructor
  * These functions creates the DccCamera objects.
  */
//@{
/*! \fn DccCamera(const u_int32_t node_id, const Card* parent)
  * \brief Constructor
  * \param node_id : specify the node_id by creating by instantiating the Node object.
  * \param parent  : pointer to the parent node
  */
  DccCamera(const u_int32_t node_id, const Card* parent);


/*! \fn ~DccCamera()
  * \brief destructor
  */
  virtual ~DccCamera();
//@}


/** \name Camera information
  *  These functions return some camera specific information.
  */
//@{
/*! \fn getFeatureHiInquiry()
  * \brief This method reads the first feature Quadlet from a DccCamera.
  * \return Quadlet : first feature Quadlet.
  */
  Quadlet getFeatureHiInquiry() {Quadlet tmp_quadlet; read(DCC_BASE+0x404, &tmp_quadlet);return tmp_quadlet;}


/*! \fn getFeatureLoInquiry()
  * \brief This method reads the second feature quadlet from a DccCamera.
  * \return Quadlet : second feature quadlet.
  */
  Quadlet getFeatureLoInquiry() {Quadlet tmp_quadlet; read(DCC_BASE+0x408, &tmp_quadlet);return tmp_quadlet;}


/*! \fn hasVideoFormat(const Videomode videomode)
  * \brief This method tests, if a videomode is supported by a Camera.
  * \return bool : true if the videomode is supported, false if not
  */
  virtual bool hasVideoFormat(const Videomode videomode) ;


/*! \fn hasFramerate(const Videomode videomode, const Framerate framerate)
  * \brief This method tests, if a framerate, with a videomode is supported.
  * \param videomode : the videomode for the framerate.
  * \param framerate : the framerate that should be tested.
  * \return bool : true, if a framerate is supported, false if not
  */
  virtual bool hasFramerate(const Videomode mode,const Framerate framerate) ;


/*! \fn int setParameter(const int buffercount, const int channel, const Isomode isomode = DEVICE_ISO_RUN, const Videomode videomode, const Framerate framerate)
  * \brief This method initialize the camera and create the iso object
  * \param isomode : DEVICE_ISO_POLL for polling mode, DEVICE_ISO_RUN
  * for free running mode at a fixed framerate, DEVICE_ISO_ONESHOT for
  * one-shot mode or DEVICE_ISO_NO for camera control only
  * \param nbuffers : number of iso-buffers (frames)
  * \param videomode : DEVICE_160x120_YUV_444, DEVICE_320x240_YUV_422,
  * DEVICE_640x480_YUV_411 or DEVICE_640x480_YUV_422.....
  * \param framerate : DEVICE_FRAMES_3_75, DEVICE_FRAMES_7_5,
  * DEVICE_FRAMES_15, DEVICE_FRAMES_30, DEVICE_FRAMES_60
  * \param channel : iso channel (a value between 1..64)
  * \return int : return a int value >= 0 if init success else < 0;
  */
  virtual int setParameter(const int buffercount = 2, const int channel = FREE_ISO_CHANNEL, const Isomode isomode = DEVICE_ISO_RUN, const Videomode videomode = DEVICE_320x240_YUV_422, const Framerate framerate = DEVICE_FRAMES_30);
//@}

/** \name Basic functions
  *  These functions implement the basic functions, defined in abstract class Device.
  */
//@{
/*! \fn start()
  * \brief This method starts the camera.
  * \return int : returns a value <=0 if an error occur, else >0
  */
  virtual int start()  ;


/*! \fn stop()
 * \brief This method stops a sending camera
 * \return int : returns a value <=0 if an error occur, else >0
 */
  virtual int stop()  ;


/*! \fn shot()
  * \brief This method sends a command to the DccCamera to send one frame.
  *
  * The command getFrame() use this function.
  */
  virtual void shot();


/*! \fn multiShot(unsigned int)
  * \brief This method enables multiShot.
  */
  virtual void multiShot(unsigned int);


/*! \fn getFrame()
  * \brief This method returns a pointer to the actual frame of a DccCamera.
  *
  *  The method blocks, until it receive a frame from video module.
  *  \return  unsigned char* : pointer to the frame
  */
  virtual Frame* getFrame() ;


/*! \fn getIsoObject() const
  * \brief This method returns the current Iso object
  */
  virtual Iso* getIsoObject() const {return isoctx;}


/*! \fn init()
  * \brief This method inits the camera to the parameter set by the function
  * setParameter() .
  *
  * You must call this method before the camera sends a frame.
  * \return L1394_SUCCESS if the init-process sucess, els L1394_FAILED. The init process
  * (normaly) fails, if the camera does not support the actual video_mode
  */
  virtual int init() ;
//@}


/** \name DccCamera control interface
  * These functions describe the interface of a camera
  */
//@{
  //! Access 'Brightness' feature register
  virtual const CameraRegister* brightness()      {return dcc_register->setOffset(0x0); }

  //! Access 'Auto-Exposure' feature CDC_register
  virtual const CameraRegister* autoExposure()    {return dcc_register->setOffset(0x4); }

  //! Access 'Sharpness' feature dcc_register
  virtual const CameraRegister* sharpness()       {return dcc_register->setOffset(0x8); }

  //! Access 'White-Balance' feature dcc_register
  virtual const CameraRegister* whiteBalance()    {return dcc_register->setOffset(0xc)->setWhiteBalance((bool)1);}

  //! Access 'White-Balance' feature dcc_register
  virtual const CameraRegister* whiteBalance(int value)    {return dcc_register->setOffset(0xc)->setWhiteBalance((bool)value);}

  //! Access 'Hue' feature dcc_register
  virtual const CameraRegister* hue()             {return dcc_register->setOffset(0x10); }

  //! Access 'Saturation' feature dcc_register
  virtual const CameraRegister* saturation()     {return dcc_register->setOffset(0x14); }

  //! Access 'Gamma' feature dcc_register
  virtual const CameraRegister* gamma()           {return dcc_register->setOffset(0x18); }

  //! Access 'Shutter' feature dcc_register
  virtual const CameraRegister* shutter()         {return dcc_register->setOffset(0x1c); }

  //! Access 'Gain' feature dcc_register
  virtual const CameraRegister* gain()           {return dcc_register->setOffset(0x20); }

  //! Access 'Iris' feature dcc_register
  virtual const CameraRegister* iris()           {return dcc_register->setOffset(0x24); }

  //! Access 'Focus' feature dcc_register
  virtual const CameraRegister* focus()           {return dcc_register->setOffset(0x28); }

  //! Access 'Temperature' feature dcc_register
  virtual const CameraRegister* temperature()    {return dcc_register->setOffset(0x2c); }

  //! Access 'Trigger' feature dcc_register
  virtual const CameraRegister* trigger()        {return dcc_register->setOffset(0x30); }

  //! Access 'Zoom' feature dcc_register
  virtual const CameraRegister* zoom()           {return dcc_register->setOffset(0x80); }

  //! Access 'Pan' feature dcc_register
  virtual const CameraRegister* pan()             {return dcc_register->setOffset(0x84); }

  //! Access 'Tilt' feature dcc_register
  virtual const CameraRegister* tilt()           {return dcc_register->setOffset(0x88); }

  //! Access 'Filter' feature dcc_register
  virtual const CameraRegister* filter()         {return dcc_register->setOffset(0x8c); }
//@}
  void setFramerate(const Framerate framerate);
protected:
  void setFramerate();
  void setVideoMode();
private:
  //! disabled copy constructor
  DccCamera(const DccCamera&);

  //iso object and csrrom
   mutable internal::DccIso *isoctx;
  internal::CsrRom* rom_info;

  //camera parameter
  mutable  int channel, frame_buffer;
  Videomode video_mode;
  Framerate frame_rate;
  Isomode   imode;

  DccRegister* dcc_register;

  mutable Quadlet tmp_quadlet;
  mutable int is_init;

  ThreadMutex mutex;
};

}    //namespace L1394
#endif
