/***************************************************************************
                          l1394_dccregister.h  -  description
                             -------------------
    begin                : Thu Aug 17 2000
    copyright            : (C) 2000-2004 by Michael Repplinger
    email                : repplinger@cs.uni-sb.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef L1394DCCREGISTER_H
#define L1394DCCREGISTER_H
#include "l1394_quadlet.h"
#include "l1394_node.h"
#include "l1394_cameraregister.h"
#include "l1394_message.h"

#define DCC_BASE                       0xFFFFF0F00000ULL

//ok
namespace L1394{

/*! \class DccRegister
  * \ingroup L1394_Register
  * \brief Class DccRegister implements the interface of class CameraRegister for DccCameras.
  *
  * As described in class CameraRegister, class DccRegister defines how to execute a command
  *  of class DccCamera .
  *
  * For example see class DccCamera or class Camera.
  *
  * Illegal combinations are ignored.
  *
  *@author Michael Repplinger
  */

class DccRegister : public CameraRegister
{

public:
/*! \name DccRegister constructor
  * These functions creates the DccRegister for DccCameras.
  */
//@{
/*! \fn DccRegister(const Node *parent)
  *  \brief constructor
  * \param Node* parent pointer to parent node
  */
  DccRegister(const Node *parent);


/*! \fn ~DccRegister()
  * \brief destructor
  */
  virtual ~DccRegister() {};
//@}

/*! \name Camera_register Interface
  * These functions implement the CameraRegister interface for DccCameras
  */
//@{
/*! \fn onePush(const bool one_push = true)  const
  * \brief This method activates or deactivates one-push mode of a feature
  * \param one_push : enable or disable one_push
  * \return bool : new status of one-push mode.
  */
  virtual bool onePush(const bool one_push = true)  const;


/*! \fn on(const bool set) const
  * \brief This method enables or disables a feature.
  *
  * After disabling a feature, no write access has effect until enabling
  * the feature again. So this method can be used, to lock (unlock) a
  * feature to a value.
  * \param set : true enables, false disables the feature.
  */
  virtual bool on(const bool on = true)             const;


/*! \fn automatic(const bool on = true)      const
  * \brief This method activates or deactivates the automode of a feature.
  * \param on : true enables, false disables the automatic mode of a feature.
  * \return bool : new status of automatic mode.
  */
  virtual bool automatic(const bool on = true)      const;


/*! \fn setValue(const int value) const
  * \brief This method changes the parameter of a feature.
  * \param value : new value for the feature.
  * \return int : value after changing the value.
  */
  virtual int setValue(const int value)             const;


/*! \fn setTriggerMode(const int value) const
  * \brief This method changes the trigger mode of a feature.
  * \param value : new value for trigger mode.
  * \return int : value after changing the trigger mode.
  */
  virtual int setTriggerMode(const int value)       const;


/*! \fn setTriggerPolarity(const bool value) const
  * \brief This method sets trigger polarity to value (False = low active)
  * \param value : new value for trigger mode.
  * \return bool : value after changing the trigger mode
  */
  virtual bool setTriggerPolarity(const bool value) const;


/*! \fn getValue() const
  * \brief This method returns the actual value of a feature.
  * \return int : actual value of the feature.
  */
  virtual int getValue()                            const;


/*! \fn getMinValue()  const
  * \brief This method returns the minimum value of a feature.
  * \return int : returns the minimum value of a feature.
  */
  virtual int getMinValue()                         const;


/*! \fn getMaxValue()  const
  * \brief This method returns the maximum value of a feature.
  * \return int : returns the maximum value of a feature.
  */
  virtual int getMaxValue()                         const;


/*! \fn hasFeature() const
  * \brief This method tests if a feature is supported by a camera.
  * \return bool : true if feature is supported, false if not.
  */
  virtual bool hasFeature()                         const;


/*! \fn hasOnePush() const
  * \brief This method tests, if one-push mode is supported by a camera
  * \return bool : true if a feature supported this mode, false if not.
  */
  virtual bool hasOnePush()                         const;


/*! \fn hasReadout() const
  * \brief This method tests for readout mode of a feature.
  * \return bool : true if a feature supports this mode, false if not.
  */
  virtual bool hasReadout()                         const;


/*! \fn hasOnOff()  const
  * \brief This method tests, if on-off mode is supported by a feature.
  * \return bool : true if a feature supports this mode, false if not.
  */
  virtual bool hasOnOff()                           const;


/*! \fn hasAuto()  const
  * \brief This method tests for auto mode of a feature.
  * \return bool : true if auto-mode is supported, false if not.
  */
  virtual bool hasAuto()                            const;

/*! \fn hasManual() const
  * \brief This method tests for manual mode of a feature.
  * \return bool : true if feature has manual support, false if not.
  */
  virtual bool hasManual()                          const;
//@}

/*! \name Some internal functions
  * These methods help to implement the interface.
  */
//@{
/*! \fn setOffset(int offset)
  * \brief This method set the offset register to access a feature.
  *
  * This method is for internal use only!!!
  * \param offset : new register offset
  * \return DccRegister* : pointer to this object.
  */
  DccRegister* setOffset(int offset) {reg_offset = offset; return this;}

/*! \fn setWhiteBalance(bool value)
  * \brief This method is for internal use only!! DON'T use it.
  *
  * This method sets the bit range for writing values to a register.
  * This method will be removed in later versions.
  * \param value : true for bit range 12..23, false for 0..11
  * \return DccRegister* : pointer to this object.
  */
  DccRegister* setWhiteBalance(bool value) {balance = value; return this;}
//@}
private:
  mutable bool balance;
  DccRegister (const DccRegister&);
  int reg_offset, sub_reg_offset;
  const Node *parent;
  mutable Quadlet data;
  Quadlet feature_hi, feature_lo;
  internal::Message* message;
};
}
#endif
