/***************************************************************************
                          l1394_device.h  -  description
                             -------------------
    begin                : Mon Jan 1 2001
    copyright            : (C) 2001-2004 by Michael Repplinger
    email                : repplinger@cs.uni-sb.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef L1394MM_DEVICE_H
#define L1394MM_DEVICE_H

#include <string>


using std::string;
#include "l1394_eventhandle.h"
#include "l1394_resource.h"
#include "l1394_frame.h"
namespace L1394{

#define FREE_ISO_CHANNEL -1

enum Videomode
{
/*Begin > For DCC Cameras*/
  DEVICE_160x120_YUV_444 = 0,
  DEVICE_320x240_YUV_422,
  DEVICE_640x480_YUV_411,
  DEVICE_640x480_YUV_422,
  DEVICE_640x480_RGB,
  DEVICE_640x480_MONO,

  DEVICE_800x600_YUV_422,
  DEVICE_800x600_RGB,
  DEVICE_800x600_MONO,
  DEVICE_1024x768_YUV_422,
  DEVICE_1024x768_RGB,
  DEVICE_1024x768_MONO,

  DEVICE_1280x960_YUV_422,
  DEVICE_1280x960_RGB,
  DEVICE_1280x960_MONO,
  DEVICE_1600x1200_YUV_422,
  DEVICE_1600x1200_RGB,
  DEVICE_1600x1200_MONO,
/*End > For DCC Cameras*/
  DEVICE_DV

};

enum Framerate
{
  //
  DEVICE_FRAMES_0 = 0,
  //! Framerate 3.75 fps24
  DEVICE_FRAMES_3_75 ,

  //! Framerate 7.5 fps
  DEVICE_FRAMES_7_5,

  //! Framerate 15 fps
  DEVICE_FRAMES_15,

  //! Framerate 30 fps
  DEVICE_FRAMES_30,

  //! Framerate 60 fps
  DEVICE_FRAMES_60,

  DEVICE_FRAMES_DV
};

enum Isomode
{
  //! Use free running isochronous receive
  DEVICE_ISO_RUN = 0,

  //! Use one-shot mode
  DEVICE_ISO_ONESHOT,

  DEVICE_ISO_MULTISHOT,

  //! No iso receive (for camera control only)
  DEVICE_ISO_NO
};

/*! \class L1394_Device
  * \ingroup L1394_Device
  * \brief This abstract class defines some basic methods for all L1394 devices.
  *
  * This class is base class for all L1394 devices and defines some basic methods, like
  * start, stop and init for the different devices.
  *
  *@author Michael Repplinger
  */
class Node;

class Device {
public:
/*! \name Device constructor
  * These functions creates the Device objects.
  */
//@{
  Device(Node* parent_node);

/*! Destructor
  */
  virtual  ~Device() {};
//@}
/** \name Event handling
  *  These methods provide access to the event handling mechanism in this library.
  */
//@{
/*! \fn  addEventHandle(L1394::EventHandle* event_handle,const Event)
  * \brief This method adds an EventHandle to a specific event
  * \param event_handle : pointer to the EventHandle object.
  * \param event : integer value for the event. (see class EventHandle )
  */
  virtual void addEventHandle(L1394::EventHandle*,const EVENT event);


/*! \fn  addEventHandle(L1394::EventHandle* event_handle)
  * \brief This method adds an EventHandle, to all events of a node.
  * \param event_handle : pointer to the EventHandle object.
  */
  virtual void addEventHandle(L1394::EventHandle*);


/*! \fn removeEventHandle(L1394::EventHandle* event_handle,const Event)
  * \brief This method removes an EventHandle from a specific event.
  *
  *  If an EventHandle object is destroyed it must be removed from all Node objects
  * \param event_handle : pointer to the EventHandle object.
  * \param event : integer value for the event. (see class EventHandle )
  */
  virtual void removeEventHandle(L1394::EventHandle*,const EVENT event);


/*! \fn removeEventHandle(L1394::EventHandle* event_handle)
  * \brief This method removes an EventHandle from any event.
  *
  *  If an EventHandle object is destroyed it must be removed from all Node objects
  * \param event_handle : pointer to the EventHandle object.
  */
  virtual void removeEventHandle(L1394::EventHandle* handle);
//@}


/** \name Device information
  *  These methods provide specific device information.
  */
//@{
/*! assign a name with this device
  */
  void setName(const string& name)                {this->name = name;}

/*! returns the device name
 */
  const string& getName()                   const {return name;}

/*! \fn getFrameSize() const
  * \brief This method returns the frame size
  * \return u_int : returns the frame size
  */
  virtual u_int getFrameSize()              const {return frame_size;}


/*! \fn getFrameWidth() const
  * \brief This method returns the frame size. (obsolete)
  *
  *  Devices supporting more than one resolution, provides normaly an init
  *  method. Before you call init, this method returns the
  *  default settings of the device.
  *
  *  \return u_int : size of a frame in bytes.
  */
  virtual u_int getFrameWidth()             const {return frame_width;}


/*! \fn getFrameHeight() const
  *  \brief This method returns the width of a frame.
  *
  *  Devices supporting more than one resolution provides normaly an init
  *  method. Before you call init, this method returns the
  *  default setting of the device.
  *  \return u_int : size of a frame in bytes.
  */
  virtual u_int getFrameHeight()            const {return frame_height;}


/*! \fn getParentNode() const
  * \brief This method returns the parent node.
  *
  * \return Node* : pointer to the parent node.
  */
  virtual const Node* getParentNode()       const {return parent_node;}


/*! \fn getDeviceType() const
  * \brief This abstract method must be implemented by all devices.
  *
  * This method is normaly for internal use. You can use it for save down_casting.
  * The following types are defined:
  *
  * return int : type of the device.
  */
  virtual DeviceType getDeviceType()         const = 0;


/*! \fn hasVideoFormat(const Videomode videomode)
  * \brief This method tests, if a videomode is supported by a Camera.
  * \return bool : true if the videomode is supported, false if not
  */
  virtual bool hasVideoFormat(const Videomode videomode)   = 0;


/*! \fn hasFramerate(const Videomode videomode, const Framerate framerate)
  * \brief This method tests, if a framerate, with a videomode is supported by the camera.
  * \param videomode : the videomode for the framerate.
  * \param framerate : the framerate that should be tested.
  * \return bool : true, if a framerate is supported, false if not
  */
  virtual bool hasFramerate(const Videomode videomode,const Framerate framerate) = 0;
//@}


/** \name Basic operations
  *  Every device must implement these basic functions.
  */
//@{
/*! \fn getFrame()
  * \brief This method returns the actual frame of a camera.
  *
  * At this time this method blocks until it gets a frame.
  * \return u_char* : pointer to the actual frame.
  */
  virtual Frame* getFrame()  ;


/*! \fn void releaseFrame(Frame* frame)
  * \brief This method releases a frame.
  *
  * You must only call this function, if the autorelease mode is disabled.
  * See also frameMode()
  * param frame : pointer to the frame.
  */
  virtual void releaseFrame(Frame* frame) ;


/*! \fn setFrameMode(bool mode)
  * \brief This method enables or disables the autorelease mode for frames.
  *
  * By default the autorelease mode is enabled. So if you call the function
  * getFrame() the last frame is released and the actual frame is returned
  * You can disable the automode but you MUST call the method releaseFrame(u_char*)
  * for every frame you get.
  * \param mode : true enables the automode, false disables the automode.
  */
  virtual void setFrameMode(const bool frame_mode) ;


/*! \fn getFrameMode() const
 *  \brief This method returns the framemode
 *  \return bool : true if the autorelease mode is enabled else false
 */
  virtual bool getFrameMode() const      ;


/*! \fn setParameter(const int buffercount, const int channel, const Isomode isomode, const Videomode videomode, const Framerate framerate)
  * \brief This method sets the device-specific parameter for iso-receive.
  *
  * Some parameter are fixed for some devices (like DV-devices) and they take no effect, if you change it.
  * \param buffercount : set how many Frame buffer are reserved
  * \param isomode     : set the isomode
  * \param channel     : set the iso-channel. A value of -1 (or FREE_ISO_CHANNEL)
  * searches for a for iso channel. Be carefull using this feature, because the used channel
  * are stored in local member variables. So if you run more programms using this library at the same
  * time, this feature can cause serious problems(if two devices uses the same iso-channel).
  *
  * The current Iso-Master don't store the used channels (or the devices don't allocate a channel) so
  * the register values for used channels are invalid.
  *
  * \param videomode   : set the video-mode
  * \param framerate   : set the framerate
  */
  virtual int setParameter(const int buffercount, const int channel, const Isomode isomode, const Videomode videomode, const Framerate framerate) = 0;

  virtual void flushIsoQueue();
/*! \fn isSending() const
  * \brief This method reports if a camera is sending or not.
  *
  * \return bool : true, if a camera is sending, false if not.
  */
  virtual bool isSending()              const  {return is_sending;}


/*! \fn init()
  * \brief This method allocates ressoures for isochronous transmission.
  *
  * If you write your own device you can use this method to allocate iso-
  * resources.
  * \return int : L1394_SUCCESS if allocating success, else L1394_Failed.
  */
  virtual int init()                           {return L1394_SUCCESS;}


/*! \fn start()
  * \brief This method starts a FireWire device and iso-receive.
  * \return int : returns L1394_FAILED if an error occur, else L1394_SUCCESS
  */
  virtual int start()                        =0;


/*! \fn stop()
  * \brief This method stops a sending device and the Iso-receive.
  * \return int : returns L1394_FAILED if an error occur, else L1394_SUCCESS
  */
  virtual int stop()                         =0;
//@}
protected:

  void setFrameSize  (const int frame_size)  {this->frame_size  = frame_size;}
  void setFrameWidth (const int frame_width) {this->frame_width = frame_width;}
  void setFrameHeight(const int frame_height){this->frame_height= frame_height;}
  void setSending    (const bool is_sending) {this->is_sending  = is_sending;}

private:
  int  frame_size;
  int  frame_width;
  int  frame_height;
  int  device_type;

  bool is_sending;

  Node* parent_node;

  string name;
};
}
#endif
