/***************************************************************************
                          l1394_deviceregister.h  -  description
                             -------------------
    begin                : Fri Nov 24 2000
    copyright            : (C) 2000-2004 by Michael Repplinger
    email                : repplinger@cs.uni-sb.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef DEVICEREGISTER_H
#define DEVICEREGISTER_H

#include "l1394_qarray.h"
namespace L1394
{

/*! \class DeviceRegister
  * \ingroup L1394_Register
  * \brief This class defines the interface of an L1394 Register for most common devices.
  *
  *  If you access a feature of a device a so called L1394 Register object is returned.
  *  The L1394 Register object provides functions to define how to execute the feature.
  *
  * The functions of this class define the minimum interface for all devices (except DccCameras).
  *
  *  If you store a pointer to a L1394 Register object remember, that the functions of
  *  this class are only valid for the last feature called by the device.
  *
  * Illegal combinations are ignored.
  *
  * In the actual version of the library this class provides only three functions : <BR>
  *
  * Function send() : <BR>
  * This function executes a command and returns the response_code of the device.
  * The response code, defined in AV/C Digital Interface Command Set General
  * Specification (Version 3.0), is used.(see the description of function send() for
  * the return values.)
  *
  * Function hasFeature() : <BR>
  * This function tests, if a feature is supported by a device.
  * It returns true, if a feature is supported, false if not.
  *
  * Function status() : <BR>
  * This function can be used, to get the actual status of a command.
  * The response code, defined in AV/C Digital Interface Command Set General
  * Specification (Version 3.0), is used.(see the description of function status() for
  * the return values.)
  *
  *
  * @author Michael Repplinger
  */

class DeviceRegister
{
public:
/*! \name DccCamera constructor
  * These functions creates the Register objects for Devices.
  */
//@{
/*! \fn ~DeviceRegister()
  *  \brief destructor
  */
  virtual  ~DeviceRegister() {}
//@}
/*! \name Register interface for all devices
  * These functions define the minimal register interface for all devices (except DccCameras)
  */
//@{
/*! \fn send() const
  * \brief This function sends a command to the device and returns the response code.
  *
  * \return int : The following response codes are defined: <BR>
  * NOT_IMPLEMENTED (0x08) : Feature not implemented. <BR>
  * ACCEPTED (0x09) : Feature is accepted and will be executed <BR>
  * REJECTED (0x0A) : The Feature is supported, but the target state does not
  * permit the execution of this command(You get this value, if you try to
  * execute the play control command and no cassette is inserted in the device.) <BR>
  * INTERIM (0x0f) : returned, if the device couldn't send a ACCEPTED or REJECTED
  * fast enough <BR>
  *
  */
  virtual  int send()            const =0;


/*! \fn hasFeature() const
  * \brief This function tests, if a command is supported by a node, false if not.
  *
  *  For this function the general inquiry command is used.
  * \return bool : return true, if node report an IMPLEMENTED for this feature, false if not.
  */
  virtual bool hasFeature()     const =0;


/*! \fn status() const
  * \brief This function returns the actual status of command.
  * \return int : The following status commands are implemented <BR>
  * NOT_IMPLEMENTED : The feature is not implemented. <BR>
  * REJECTED : The device implements the status command, but device state doesn't
  * permit a status of the command. <BR>
  * IN_TRANSITION : The device is already in transition with the status command <BR>
  * STABLE : status command was executed successful<BR>
  */
  virtual L1394::QArray status()          const =0;
//@}
};
}
#endif
