/***************************************************************************
                          l1394.h  -  description
                             -------------------
    begin                : Tue Aug 22 2000
    copyright            : (C) 2000-2004 by Michael Repplinger
    email                : repplix@author_email=repplix@studcs.uni-sb.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef L1394Event_H
#define L1394Event_H

#include "l1394_eventhandle.h"
#include "l1394_message.h"
#include "l1394_resource.h"

namespace L1394{
/*! \class Event
  *  \ingroup L1394_Event
  * \brief This class reports changes on the FireWire bus.
  *
  * Every Node object has an Event object, where you can add some EventHandle to
  * get information about incoming events on the FireWire bus. In the actual version
  *  of this library 5 events are defined.
  *
  * Event BUSRESET: <BR>
  * This event calls the method busreset() on the connected EventHandle objects.
  *  This event happens only to Card objects.
  *
  * Event NODE_DISABLED:<BR>
  * This event calls the method nodeDisabled() on the connected EventHandle objects.
  * This event occurs, if a Node Object is disconnected from FireWire Bus and stored
  * in the inactive_node_list of the global NodeContainer object. Connected EventHandle
  * objects
  *
  * Event NODE_ENABLED: <BR>
  * This event call the method nodeEnabled() on the connected EventHandle objects.
  * This event occur, if a Node Object is reconnected from FireWire Bus and removed
  * from the inactive_node_list of the NodeContainer object.Node objects, that connect
  * to this event get also the NODE_DISABLED event.
  *
  *  Event NODE_DESTROY:<BR>
  *  This event call the method nodeDestroy() on the connected EventHandle objects.
  * This event happens if the destructor of a Node object is called, what (should ;) )
  * only happen when a FireWire node is disconnected from the bus.
  *
  * See the tutorial for examples.
  *
  *@author Michael Repplinger
  */
class Node;

class Event{

public:
/*! \fn Event(const Node *, const Card*)
  * \brief Constructor
  */
  Event(const L1394::Node *, const L1394::Card* = 0);

/*! \fn ~Event()
  * \brief Destructor
  */
  ~Event();
/*! \name Functions to add/remove EventHandles
  * These functions add or remove EventHandles from this object.
  */
//@{
/*! \fn addEventHandle(EventHandle* event_handle,const Event )
  * \brief This method adds an EventHandle to a specific event.
  *
  * See description of class Event for more information.
  *
  * \param event_handle : pointer to the EventHandle object
  * \param e : event that should be handled
  */
  void addEventHandle(EventHandle* event_handle, const EVENT event);

/*! \fn addEventHandle(EventHandle* handle)
  * \brief This method adds an EventHandle to all events of a node.
  *
  * \param event_handle : pointer to the EventHandle object
  */
  void addEventHandle(EventHandle* event_handle);

/*! \fn removeEventHandle(EventHandle* handle,const Event )
  * \brief With this method you can remove an EventHandle from a specific event.
  *
  * All EventHandle must removed, if the EventHandle is deleted.
  * (So you can call this method in the destructor of your EventHandle.)
  * \param handle : pointer to the EventHandle object
  * \param e : event that should be handled
  */
  void removeEventHandle(EventHandle* event_handle, const EVENT event);

/*! \fn removeEventHandle(EventHandle* handle)
  * \brief This method removes an EventHandle from all events.
  *
  * All EventHandle must removed, if the EventHandle is deleted.
  * (So you can call this method in the destructor of your EventHandle.)
  * \param event_handle : pointer to the EventHandle object
  */
  void removeEventHandle(EventHandle* event_handle);
//@}

//@{
/*! \fn call(const Event)
  * \brief This method emit an event.
  * \param int Integer value signal that specify the signal
  * \param Node* pointer to the object that call this function
  */
  void call(const EVENT event);
//@}
private:
  list <EventHandle*> busreset_list;     //list of EventHandle object connected to BUSRESET event
  list <EventHandle*> node_destroy_list; //list of EventHandle object connected to NodeDestroy event
  list <EventHandle*> node_disabled_list;
  const L1394::Node* parent;                          //pointer to parent object that call the events
  const L1394::Card* card;
  internal::Message* message;
};

}
#endif
