/***************************************************************************
                          l1394_eventhandle.h  -  description
                             -------------------
    begin                : Tue Oct 24 2000
    copyright            : (C) 2000-2004 by Michael Repplinger
    email                : repplinger@cs.uni-sb.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef L1394EVENTHANDLE_H
#define L1394EVENTHANDLE_H



#include <list>
#include <iostream>
#include <iterator>


namespace L1394{
/*! \class EventHandle
 *  \ingroup L1394_Event
 *  \brief This class represents the eventhandle to get information about an event.
 *
 *  The idea of this class is, to inherit external classes from this class,
 *  to get information about changes on FireWire bus. This Class provides functions for
 *  every defined event. After creating an EventHandle you can add it to every Node
 *  (see example).<BR>
 *  By default all functions do nothing and only the interest functions must be re-implemented.
 *  At this time there are five functions (and events) that can be used: <BR>
 *
 *  Method busreset(const Card*): <BR>
 *  This method is called if a busreset occur. This method is only called from
 *  Card objects.
 *
 *  Method nodeDisabled(const Node*): <BR>
 *  This method is called, if a node is disconnected form the bus. The Node object
 *  is moved in the inactive_node_list, but not deleted.
 *
 *  Method nodeEnabled(const Node*): <BR>
 *  This method is called, if a known node is reconnect to the bus. A known node,
 *  is a node which object is in the inactive_node_list.
 *
 *  Method nodeDestroy(const Node*): <BR>
 *  This method is called, if the destructor of a Node object is called. After
 *  a defined time interval a Node object in the inactive_node_list is deleted.
 *
 *  A little example show the idea
 *  \code
 *  #include <l1394_session.h>
 *  using namespace l1394;
 *  class MyEventHandle : public EventHandle
 *  {
 *    MyEventHandle(Node* node) {my_node = node; my_node->addEventHandle(this);} //connect this handle to all events
 *    ~MyEventHandle()  {  if (my_node != NULL) my_node->removeEventHandle(this);}
 *
 *    void busreset(const Card* n) { cout << "Bus reset" << endl; } //occur only, if my_node is a Card
 *    void nodeDisabled(const Node* n) { cout << "Node disconnected from bus "<< endl;}
 *    void nodeEnabled(const Node* n) {cout << "Node is back on bus" << endl;}
 *    void nodeDestroy(const Node* n) {cout << "Object is deleted " << endl; my_node = NULL;}
 *    private:
 *    Node* my_node;
 * }
 *
 *  int main(int argc, char* argv[])
 *  {
 *    Session *session = GetSession();
 *    Card* card = session->getCard(0);
 *    MyEventHandle* event_handle = NULL;
 *    if (card != NULL)
 *      event_handle = new MyEventHandle(card); //Now you get Information
 *
 *    delete event_handle;
 *    delete session;
 *  }
 *  \endcode
 * @author Michael Repplinger
 */


class Node;
class Card;
class EventHandle
{
public:
/*! \name EventHandle constructor
  * These functions creates the EventHandle objects.
  */
//@{
/*! \fn EventHandle()
  * \brief Constructor
  */
  EventHandle() {};

/*! \fn ~EventHandle()
  * \brief Destructor
  */
  virtual ~EventHandle();

//@}
/** \name Functions that are called by an event
  *  These functions can be re-implement to handle a certain event.
  */
//@{
/*! \fn busreset(const Card* node)
  * \brief This method handles the BUSRESET event.
  *
  * Note that this event is only called by a Card object.<BR>
  *  For a example see tutorial.
  * \param node : pointer to the Node object, that call this event.
  */
  virtual void busreset(const Card*) {};


/*! \fn nodeDestroy(const Node* node)
  * \brief this method handles the NODE_DESTROY event.
  *
  * This event is called, if the destructor of a Node is called.<BR>
  *  For a example see tutorial.
  * \param node : pointer to the Node object, that call this event.
  */
  virtual void nodeDestroy(const Node* ) {};


/*! \fn nodeDisabled(const Node* node)
  * \brief This method handles the NODE_DISABLED event
  *
  * This event is called, if a FireWire node is disconnected from bus
  * and the L1394 node object is inserted in the inactive_node_list.<BR>
  * For example see tutorial.
  * \param node : pointer to the Node object, that call this event.
  */
  virtual void nodeDisabled(const Node* ) {};


/*! \fn nodeEnabled(const Node* node)
  * \brief This method handles the NODE_ENABLED event
  *
  * This event is called, if the Node object is removed from the
  * inactive_node_list and insert in the active_node_list again.<BR>
  * For example see tutorial.
  * \param node : pointer to the Node object, that call this event.
*/
  virtual void nodeEnabled(const Node* ) {};
//@}
private:
  //disabled copy constructor
  EventHandle(const EventHandle&);
};
}
#endif
