/***************************************************************************
                          l1394_fcpnode.h  -  description
                             -------------------
    begin                : Tue Oct 24 2000
    copyright            : (C) 2000-2004 by Michael Repplinger
    email                : repplinger@cs.uni-sb.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef L1394FCPNODE_H
#define L1394FCPNODE_H

#include "l1394_node.h"
#include "l1394_device.h"
#include "l1394_dviso.h"

namespace L1394{

/*! \class FcpNode
  * \ingroup L1394_Node
  * \brief This class represents a node using the FCP protocol to control a device.
  * (normally AV/C nodes)
  *
  * A FcpNode stores up to 8 subunits in an array. The subunit itself
  * stores a pointer to a specific device.(see also class Subunit).<BR>
  * This class implements also functions to send commands to a node and gets
  * information about the Unit and subunits.
  *
  * For more information about FCP protocol see IEEE specification.
  * For more information about AV/C see 'AV/C Digital Interface Command Set
  * General specification'.
  *@author Michael Repplinger
  */

class FcpNode : public Node  {

public:
/*! \name FcpNode constructor
  * These functions creates the FcpNode objects.
  */
//@{
/*! \fn FcpNode(const u_int32_t node_id, const Card* parent)
  * \brief Constructor
  * \param node_id : the current node-id by instantiating the object
   * \param parent  : pointer to the parent_node
  */
  FcpNode(const u_int32_t node_id, const Card* parent);

/*! \fn ~FcpNode()
  * \brief Destructor
  */
  virtual ~FcpNode();
//@}

/*! \name Fcp specific transactions and functions
  *  These functions send a command, based on the
  *  FCP protocol, to a FcpNode and return the response.
  */
//@{
/*! \fn send(const Quadlet& command) const
 *  \brief This method sends a Quadlet (4 bytes) to this node using FCP protocol.
 *  \param command  : Quadlet with the command
 *  \return Quadlet : The response quadlet.
 */
  Quadlet send(const Quadlet& command) const;


/*! \fn send(const QArray& command) const
  * \brief This method sends an array of Quadlets to this node, using FCP protocol.
  * \param command : QArray with the extended command
  * \return QArray : The response QArray.
  */
  QArray send(const QArray& command) const;


/*! \fn decode(const QArray& value) const
  * \brief This method decodes a response and returns the response-code as string.
  * \param value  : QArray that should be decoded
  * \return char* : pointer to the response
  */
  const  char* decode(const QArray& value) const {return decode(value.getQuadlet(0));};


/*! \fn decode(const Quadlet& value) const
  * \brief This method decodes a response and returns as string.
  * \param value  : Quadlet that should be decoded
  * \return char* : pointer to the response
  */
  const char* decode(const Quadlet& value) const;
//@}


/** \name Unit/Subunit information
  *  These functions return some Unit and Subunit informations.
  */
//@{
/*! \fn getSubunitCount() const
  * \brief This functions return the count of subunits.
  * \return int : integer value with subunit count
  */
  int getSubunitCount() const {return subunit_count;}


/*! \fn getSubunitInfo(int subunit)
  * \brief This method returns the subunit information from the i-th subunit.
  * \param subunit : integer value for the subunit
  * \return QArray : with the subunit information.
  */
  QArray getSubunitInfo(int subunit);


/*! \fn getUnitInfo()
  * \brief This method returns the unit information block.
  * \return QArray : returns the unit information block
  */
  QArray getUnitInfo();


/*! \fn findSubunit(const int subunit_type) const
  * \brief This method returns the subunit with a specific type.
  *  \param subunit_type : integer value with the subunit_type
  * \return Subunit* : pointer to Subunit object with type subunit_type, NULL if no Subunit
  * with this type exist.
  */
  Device* findSubunit(const int subunit_type) const;


/*! \fn getDeviceArray() const
  * \brief This method return an array with size 8.
  *
  *  The array contain pointer to the subunits of an AvcNode. If an AvcNode has
  * less than 8 subunits the array is filled with NULL.
  * \return Subunit** : pointer to the Subunit array
  */
  Device** getDeviceArray() const {return device_array;}


/*! \fn getIsoObject() const
  * \brief This method returns a DvIso-object
  * \return Iso* : pointer to the ISO-object, or NULL.
  */
  Iso* getIsoObject()       const {return dv_iso;}
//@}

private:

//disabled Copy constructor
  FcpNode(const FcpNode&);

/*! \fn createSubunitList()
  * \brief This method create the subunits
  */
  void createSubunitList();

  internal::CsrRom *rom_info;

//Device **device_array;
  Device** device_array;
  static const char* subunit_type[];

  u_int32_t subunit_count;

  internal::DvIso* dv_iso;
  raw1394handle_t dv_iso_handler;
};
}
#endif
