/***************************************************************************
                          l1394_node.h  -  description
                             -------------------
    begin                : Wed Jul 5 2000
    copyright            : (C) 2000-2004 by Michael Repplinger
    email                : repplix@studcs.uni-sb.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#ifndef L1394NODE_H
#define L1394NODE_H


#include <netinet/in.h>
#include <sys/poll.h>
#include <sys/types.h>
#include <sys/time.h>
#include <unistd.h>
#include <string>
#include "libraw1394/raw1394.h"
#include "libraw1394/csr.h"
// new lib L1394

#include "l1394_resource.h"
#include "l1394_qarray.h"
#include "l1394_event.h"
#include "l1394_csrrom.h"
#include "l1394_nodefactory.h"
#include "l1394_nodecontainer.h"
#include "l1394_message.h"
#include "l1394_iso.h"

namespace L1394
{

class Device;
class Card;

enum NodeState{ENABLED = 0, DISABLED};

/** \class Node
  * \ingroup L1394_Node
  * \brief Every node connected to the FireWire bus is  represented as a Node object.
  *
  * This class is base class for all Nodes and implements methods for the three basic transactions
  * read, write and lock.
  * You can also get information about a node, like GUID and the actual node_ID, or card_ID.
  *
  * This class provides also an reference counter to allow connecting a Node to more
  * than one card in the same PC.
  * Every node has an Event object, that can be used to get information
  * about BUSRESET and NODE_DESTROY events. (See the documentation of class
  * EventHandle )
  *
  * This class is inherited by:
  *
  *  Card :
  * A Card represents a FireWire card. The Card manages the
  * Node objects for FireWire nodes connected to a FireWire card.
  * For more information see the documentation of class Card .
  *
  * FcpNode :
  * A FcpNode represents a FireWire node using FCP protocol.
  * Normally a FcpNode represents an AV/C device, with up to 8 Subunits.
  * For more information see the documentation of class FcpNode .
  *
  * DccCamera :
  * A DccCamera represents a camera using DCC-protocol, like Sony DFW-V500
  *  and DFW-VL500 cameras.
  * For more information see documentation of class DccCamera .
  *
  * PhyNode :
  * A PhyNode represents a FireWire node, with a physical layer only(like hubs).
  * For more information see the documentation of the class PhyNode .
  *
  *@author Michael Repplinger
  */

class Node
{
public:
/*! \name Node constructor
  * These functions creates a Node.
  */
//@{
/*! \fn Node(const u_int32_t node_id, const Card* parent)
  * \brief constructor
  * \param node_id : the actual node_id by instantiating the Node object.
  * \param parent  : pointer to the parent-node
  */
  Node(const u_int32_t node_id, const Card* parent);


/*! \fn ~Node()
 *  \brief destructor
 */
  virtual ~Node();
//@}


/*! \name Transactions
  *  These methods implement the three basic asynchrone transaction (read, write and lock), defined in the
  *  FireWire standard.
  */
//@{
/*! \fn  read(const u_int64_t address, Quadlet* response) const
 *  \brief This method reads 4 bytes (a Quadlet) from the CsrRom and stores the value in response
 *
 *   For information about the structure of the CsrRom see IEEE-1212 specification.
 *  \param address : u_int64_t value for the CSR-address.
 *  \param response : pointer to the response Quadlet
 *  \return int : L1394_FAILED, if the transaction fails, else L1394_SUCCESS.
 */
  virtual int read(const u_int64_t address, Quadlet* response)      const;


/*! \fn  write(const u_int64_t, const Quadlet& data) const
 *  \brief This method writes 4 bytes (a Quadlet) to CsrRom.
 *  \param address : u_int64_t value that specify the address in CSR.
 *  \param data : Quadlet with the data to write in the CsrRom.
 *  \return int : L1394_FAILED, if the transaction fails, else L1394_SUCCESS.
 */
  virtual int write(const u_int64_t address ,const Quadlet& data )  const ;


/*! \fn lock   (const u_int64_t address, const Quadlet& data, const unsigned int extcode,const Quadlet& argument, Quadlet* result) const
  * \brief This method realizes an atomic transaction on the FireWire bus.
  *
  * An atomic transaction means, that no node can access the address during the lock operation.
  * \param address : u_int64_t value for the CSR-address
  *  \param data : The data Quadlet defines the value that should be executed on the address
  * \param extcode : The extension code defines the type of lock operation.
  *  \param argument : The argument value is compared with the actual address value and depending on the extension code a operation with the data value is executed.
  *  \param result : If a value is returned, it is stored in result
  *  \return int: L1394_FAILED, if the transaction fails, else L1394_SUCCESS.
  */
  virtual int lock(const u_int64_t address,const Quadlet& data, const unsigned int extcode, const Quadlet& argument, Quadlet* result) const;
//@}


/** \name Node information
  *  These functions report actual information about the status of a Node.
  */
//@{
/*! \fn getNodeID() const
 *  \brief This method returns the current node_ID of a node.
 *
 *  \return u_int32_t : value with the actual node ID.
 */
  inline u_int32_t getNodeID() const {return __node_id;}


/*! \fn getCardID() const
 *  \brief returns the card_id of the parent node
 *  \return u_int32_t : value with card number
 */
  u_int32_t   getCardID()      const ;


/*! \fn getParentCard() const
  * \brief This method returns a pointer to the current parent node
  */
  inline const Card* getParentCard()  const {return __parent;}


/*! \fn getNodeType() const
  * \brief returns the type of a node (coded as integer value)
  *
  * \return NodeType : value with the node type.
  */
  inline NodeType getNodeType()       const {return __node_type;}


/*! \fn getGuid() const
 *   \brief This method returns the GUID of a node.
 *  \return u_int64_t  : value with the guid
 */
  inline const u_int64_t& getGuid()   const {return __guid;}


/*! \fn isPhyOnly() const
  * \brief This method tests, if a node has only a physical layer,
  *  like hubs.
  * \return true, if node has only a physical layer, else false.
  */
  virtual bool isPhyOnly()            const {return false;}


/*! \fn getState() const
  * \brief This method reports the state of a node.
  *
  *  A node has three states. L1394_ENABLED, if the node is connected to the
  *  FireWire bus, L1394_DISABLED, if the node is not connected to the bus
  *
  * \return int : L1394_ENABLED, if the node is connected to the bus, else L1394_DISABLED.
  */
  inline int getState()           const {return __state;}

/*! \fn getRefCount() const
  * \brief This method returns the current ref-counter.
  */
  inline int getRefCount()        const {return __reference_counter;}


/*!  returns the node_type as const char* */
  static const char* char_node_type[];


/*! \fn getEventObject() const
  * \brief This method returns the event object.
  */
virtual Event* getEventObject() const {return __event;}


/*! \fn getIsoObject() const
  * \brief This method returns the Iso-Object, NULL if the Device is not ISO-Capable,
  *  or configured.
  */
virtual Iso*   getIsoObject()   const {return 0;}

/*! \fn getDeviceList() const
  * \brief This method returns the list of device-interfaces provided by a Node.
  */
const list<Device*>& getDeviceList()  const  {return device_list;}
//@}

/** \name internal Node methods
  *  These methods are for INTERNAL USE ONLY!!!! Don't touch any of this methods or no correct work is possible.
  */
//@{
/*! \fn increaseRefCount()
  * \brief increases the reference counter (for internal use only) DON'T call this function!!!
  */
  int increaseRefCount() ;


/*! \fn decreaseRefCount()
  * \brief decreases the reference counter (for internal use only)  DON'T call this function!!!
  */
  int decreaseRefCount();


/*! \fn setNodeID(const u_int32_t node_number)
 *  \brief This method sets the current node-ID. (for internal use only!!)
 */
  void setNodeID(const u_int32_t node_id)   { __node_id = node_id;}

/*! These method sets the current parent object. (for internal use only!!)
 */
   void setParentCard(const Card* parent)   { __parent = parent;}
//@}

protected:
/*! This method adds a Device object to the device list. You are responsible
  * to delete the pointer in this list.
  */
  void addDevice(Device* device) {device_list.insert(device_list.end(), device);}

/*!\fn readGuid()
 * \brief This method reads the guid of a nodesetState
 */
  void readGuid();


/*! \fn setState(const NodeState s)
  * \brief This method sets the state of a node.
  *
  *  This method is for internal use only. If you inherit from this class
  * don't use this method or no correct work is possible.
  */
  void setState(const NodeState s);


/*! \fn setNodeType(const NodeType node_type)
  * \brief This method sets the node_type
  */
  void setNodeType(const NodeType& node_type)   {__node_type = node_type;}


/*! \fn setGuid(const u_int64_t guid)
  * \brief This method sets the GUID
  */
  void setGuid(const u_int64_t& guid)           {__guid = guid;}

/** change the Eventobject.
  */
  void setEvent(Event* event);

/* The global Message object */
  internal::Message* message;

  //! global bus reset counter
  static u_int32_t *reset_counter_array;

private:

  //! The Global Unique Identifyer of a node
  u_int64_t __guid;

  //! User defined name of node
  string __name;

  //! The type of a node
  NodeType __node_type;

  //! The event object for a node
  Event *__event;

  //reference counter
  int __reference_counter;

  NodeState   __state;   /* state of a node */

  u_int32_t   __node_id; //! actual node_ID

  const Card* __parent;  //! actual node_ID

  list<Device*> device_list;
};

} //end namespace l1394

#endif
