/***************************************************************************
                          l1394nodecontainer.h  -  description
                             -------------------
    begin                : Fri Sep 8 2000
    copyright            : (C) 2000-2004 by Michael Repplinger
    email                : repplinger@cs.uni-sb.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef L1394NODECONTAINER_H
#define L1394NODECONTAINER_H

#include "l1394_message.h"
#include "Thread.hpp"
#include <map>


using std::map;

namespace L1394{
class Node;
namespace internal{
/*! \class NodeContainer
  * \ingroup Internal
  * \brief This class stores and manages the Node objects.
  *
  * A NodeContainer object stores the Node objects, created by the Card objects
  * <BR>
  * To store the objects two (STL)single associated container are used.The
  * Node objects are stored as key/value pairs, for fast access to them.
  * The key is the GUID, the value the object pointer. <BR>
  * The first container stores the active Node objects, that are connected to FireWire bus
  * The second one stores the inactive Node objects. An inactive node object is a Node
  * object, that was disconnected from FireWire bus, but was not deleted.
  * After a defined timeout, node objects for disconnected nodes are deleted.
  *
  * @author Michael Repplinger
  */

class NodeContainer {


public:

  static NodeContainer* getNodeContainer();

/*! \fn ~NodeContainer()
  * \brief destructor
  */
  ~NodeContainer();


/*! \fn getNode(u_int64_t guid)
  * \brief This method returns the Node object with guid, NULL if this node not exist.
  *
  *  If you use this function, the internal reference counter is increased.
  * \param guid : u_int64_t value with the guid
  * \return Node* : pointer to the Node object with guid value, NULL if the Node object not exist
  */
  Node* getNode(u_int64_t guid) ;


/*! \fn findNode(u_int64_t guid)
  * \brief This method returns the Node object with guid, NULL if this node not exist.
  *
  * The difference between getNode(u_int64_t guid) is, that the reference counter
  * is not be increased. This method is for internal use only.
  * \param guid : u_int64_t value with the guid
  * \return Node* : pointer to the Node object with guid value, NULL if the Node object not exist
  */
  Node* findNode(u_int64_t guid);


/*! \fn nodeExist(u_int64_t guid)
  * \brief This method checks, if a Node object with a specific guid exist.
  * \param u_int64_t value for the guid
  * \return bool : This method return true, if a Node object with guid exist, else false.
  */
  bool nodeExist(u_int64_t guid);


/*! \fn release(Node* node)
  * \brief This method releases Node objects.
  *
  *  You must use this function, if you get a Node object with getNode(u_int64_t guid).
  * This method decrease the reference counter and if the reference counter is less than
  * 1, the Node object is inserted in the inactive_node_list.
  * \param node : pointer to the Node object.
  */
  void release(Node* node);


/*! \fn insert(Node* node)
  * \brief This method inserts a new Node object into the node_list.
  *
  * If a Node object with this guid exist, the Node will not be inserted in
  * the node_list.
  * \param node   : pointer to Node object that should be inserted.
  */
  void insert(Node* node);

  void removeNode(const Node* n);

protected:
/*! \fn NodeContainer()
  * \brief Constructor
  */
  NodeContainer();

private:
  //map for active nodes
  map <u_int64_t, Node*> node_list;

  //map for inactive nodes
  map <u_int64_t, Node*> inactive_node_list;

  Message*  message;
  static  NodeContainer* container;
  static  ThreadMutex mutex;
};
}
}
#endif
