/***************************************************************************
                          l1394_nodefactory.h  -  description
                             -------------------
    begin                : Wed Jan 31 2001
    copyright            : (C) 2001-2004 by Michael Repplinger
    email                : repplix@studcs.uni-sb.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef L1394_NODEFACTORY_H
#define L1394_NODEFACTORY_H

#include "l1394_avcvcr.h"
#include "l1394_avccamera.h"
#include "l1394_message.h"
#include "Thread.hpp"
#include "libraw1394/raw1394.h"

namespace L1394{

class Card;
class FcpNode;
class Node;
namespace internal{
/*! \class NodeFactory
  *  \brief This class creates L1394 node objects.
  *
  *  This class creates the default L1394 node objects  (Crad, DccCamera, FcpCamera and PhyNode)
  *  and devices (AvcCamera, AvcVcr..). If you like to use your own L1394 Node
  *  objects inherit from this class and rewrite the createNode(), createCard() and
  *  createDevice() function.
  *
  *  This class is designed as singelton. So if you use your own NodeFactory you must
  *  instantiate the new factory before the Session object. The Session object calls
  * the method getFactory() and gets the new factory.
  *
  *@author Michael Repplinger
  */

class NodeFactory {
public:

/*! \fn getNodeFactory()
  * \brief This static method returns the pointer to the NodeFactory
  *
  * \return NodeFactory* : pointer to the NodeFactory.
  */
  static NodeFactory* getNodeFactory();


/*! \fn ~NodeFactory()
  * \brief destructor
  */
  virtual ~NodeFactory();


/*! \fn Node* createNode(Card* card, const u_int32_t node_id)
  * \brief This method creates the node-objects.
  *
  * If you like to create your own Node objects inherit from this class and
  *  rewrite this function.
  *  \param card : pointer to the Card object.
  *  \param node_id : node_id of the new node.
  *  \return Node*: pointer to the Node object.
  */
  virtual Node* createNode(Card*, const u_int32_t);


/*! \fn createCard(const u_int32_t card_id, const u_int32_t node_id, raw1394handle_t default_handle)
  * \brief This method instantiates the Card object.
  *
  * \param card_id : The card-id of the card.
  * \param node_id : The node-id of the card.
  * \param default_handle : pointer to the default_handle for the new card.
  * \return Card* : pointer to the Card object.
  */
  virtual Card* createCard(const u_int32_t card_id, const u_int32_t node_id, raw1394handle_t default_handle);

/*! \fn createDevice(FcpNode* parent_node, int subunit_value)
  * \brief This method instantiates the Device objects for FcpNodes.
  * \param parent_node : pointer to the parent_node.
  *  \param subunit_value : The subunit_value defines the subunit_type and subunit_id.
  */
  virtual Device* createDevice(FcpNode*, int subunit_type);

  protected:

  NodeFactory();

/*! \fn scanNode(Card* card, unsigned int node_id)
 *  \brief scans a specific node and returns the type (AVC, DCC, ...) as integer
 *  value (defined in class Node)
 *  \param Card* card: pointer to the card object.
 *  \param node_id: the actual node-id on the bus.
 *  \return int : returns the node type as integer value.
 */
  virtual NodeType scanNode(Card* c, unsigned int node_id);

private:
  static  NodeFactory* factory;
  static  Message* message;
  static  ThreadMutex mutex;
};

}//end namespcae internal
}//end namespace l1394
#endif
