/***************************************************************************
                          l1394_qarray.h  -  description
                             -------------------
    begin                : Mon Jun 26 2000
    copyright            : (C) 2000-2004 by Michael Repplinger
    email                : repplix@studcs.uni-sb.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef _L1394_QARRAY_H
#define _L1394_QARRAY_H

#include <iostream>
#include "l1394_quadlet.h"
namespace L1394{


/*! \class QArray
  * \brief A QArray represents an dynamic array of Quadlets.
  *
  * Like Quadlets, QArrays are one of the basic data types in this library
  * to communicate with FireWire nodes. If you send more than 4 bytes
  * to a FireWire node you should use an QArray.<BR>
  * The QArray stores also information about the size of the array. It provides
  * most of the functionality like a Quadlet (without the compute operators).
  *
  * This class also provides functions to convert a QArray to an integer array
  * or a char array.
  *
  * @author Michael Repplinger
  */

class QArray
{

public:
/** \name Constructors
  *  These are the different constructors to create a QArray.
  */

//@{
/*! \fn QArray(int size)
 *  \brief Create a Qarray with a certain size.
 *  \param size : Set the size of a QArray.
 */
  QArray(int size);


/*! \fn QArray(const QArray&)
  * \brief Copy constructor
  * \param qarray : reference to the QArray
  */
  QArray(const QArray& qarray);


/*! \fn ~QArray()
 *  \brief Destructor
 */
  ~QArray();

//@}

/** \name Manipulate a QArray
  *  These functions manipulate the values stored in a QArray.
  */
//@{
/*! \fn append(const QArray& qarray)
  * \brief This method appends an QArray to the end of a QArray.
  * \param qarray : array to append.
  */
  void append(const QArray& qarray);

/*! \fn insert(const Quadlet value, unsigned int position)
  * \brief This method inserts a quadlet at a specific position in the array and
  *  overwrites the old value.
  *
  * If you try to insert a value to an invalid position, the value will not be set.
  * \param value : Quadlet to insert
  * \param position : unsigned integer value for the position
  */
  void insert(const Quadlet value, const unsigned int position);


/*! \fn insert(const u_int32_t value, const unsigned int position)
  * \brief This method inserts am u_int32_t value at a specific position in the array and overwrites the old value.
  *
  * If you try to insert a value to an invalid position, the value will not be insert.
  * \param value : u_int32_t Integer to insert
  * \param position : unsigned integer value for the position.
  */
  void insert(const u_int32_t value, const unsigned int position);


/*! \fn getQuadlet(unsigned int i) const
 *  \brief This method returns the i'th Quadlet in an array.
 *
 *  Valid values are from 0 to length-1. If you try to get a Quadlet from position >= length,
 *  a Quadlet with value 0 is returned
 *  \param i : integer value for the position
 *  \return Quadlet : the i'th Quadlet. If i>= size a empty Quadlet is returned
 */
  Quadlet getQuadlet(const unsigned int i) const {if (i<size) return qarray[i]; return Quadlet();}


/*! \fn getByte(const unsigned int i)  const
  * \brief This method returns the i'th byte of the array.
  *
  * The range of i depends on the size of your QArray and is between 0<= i < size*4.
  * Byte 0 are the 8 most significant bits of the array
  * \return int : integer value of the i'th byte of the array.
  */
  int getByte(const unsigned int i) const { return qarray[i/4].getByte(i%4);}


/*! \fn setByte(const unsigned int i, const unsigned int value)
  * \brief This method sets byte i of the QArray to a specific value.
  *
  * The range of i depends on the size of your QArray and is between 0<= i < size*4.
  * \param i : unsigned integer for the position
  * \param value : unsigned integer for the value
  */
  void setByte(const unsigned int i, const unsigned int value);
//@}

/** \name Convert a QArray
  *  These functions convert a QArray to another basic type.
  */

//@{
/*! \fn toIntArray(u_int32_t*) const
 *  \brief This method converts a QArray to an integer array (for compatibility use only)
 *
 *  The integer array has the same size like the QArray. The programmer is responsible to delete this array.
 *  \return u_int32_t* : pointer to the integer array.
 */
  void toIntArray(u_int32_t* ) const;


/*! \fn toCharArray(u_char*) const
  * \brief This method converts the value of a QArray to an u_char array.
  *
  * The size of the u_char array is size of the QArray * 4. Note, that the programmer
  * are responsible to delete this array.
  * \return u_char* : pointer to the u_char array.
  */
  void toCharArray(u_char* ) const;


/*! \fn getSize()  const
 *  \brief This method returns the size of a QArray.
 *  \return int : the size of the QArray
 */
  int getSize() const { return size;}
//@}


/*! \fn operator<<(ostream& output, const QArray& qarray)
 *  \brief This method prints the QArray to ostream
 *  \param output : reference to the output stream.
 *  \param qarray : reference to the QArray.
 *  \return ostream& : reference to the output ostream.
 */
  friend ostream& operator<<(ostream& output, const QArray& array);

/** \name Some operators
  *
  */
//@{

/*! \fn operator= (const QArray& qarray)
  * \brief This method sets the value of this QArray to the value of qarray.
  *
  * If this QArray is bigger than qarray, the last Quadlets are unchanged.
  * If this QArray is smaller, this QArray will be resized to the size of qarray.
  */
  QArray& operator= (const QArray&);


/*! \fn operator== (const QArray& qarray)
  * \brief This method compare two QArrays.
  *
  * Two QArrays with equal size are equal, if all their Quadlets are equal. Two
  * QArrays with different size are equal, if all Quadlets of the smaller QArray
  * are equal to the bigger one.
  * \return bool : return true if two QArrays are equal, false if not.
*/
  bool operator== (const QArray&) ;

//@}
private:

/*! \fn resize(const u_int factor)
  *  \brief This method resizes the QArray
  */
  void resize(const u_int factor);

  unsigned int size, internal_size;
  Quadlet *qarray;

};
}
#endif
