
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef _L1394_QUADLET_H
#define _L1394_QUADLET_H

#ifndef TRUE
#define TRUE true
#endif

#ifndef FALSE
#define FALSE false
#endif


#include <string>

using std::string;
using std::ostream;
using std::istream;

namespace L1394{
/*! \class Quadlet
  * \brief This class stores a Quadlet and provides functions to manipulate it.
  *
  *  A Quadlet is one of the base data-types of this library and represents
  * the atomic unit of information that can be read or write to a FireWire node.
  *  The Quadlet stores 4 byte and is realized as an char array.<BR>
  * This class provides also a lot of functions to manipulate the bits and
  * bytes.
  *
  *  The location 31 is msb, location 0 is lsb.
  *
  * @author Michael Mirold, Michael Repplinger
  */

class Quadlet {


public:

/** \name Constructors
  *  These are the different constructors to create a Quadlet.
  */

//@{
/*! \fn Quadlet()
  * \brief Constructs an empty quadlet.
   */
  Quadlet();

/*! \fn Quadlet( const char *data )
  * \brief Constructs an quadlet from a char array.
  * \param value: const char pointer on an char array containing the data.
  */
  Quadlet( const char* );

/*! \fn Quadlet( const u_int32_t data )
  * \brief Constructs an quadlet from an int.
  * \param data : Integer value representing the data.
  */
  Quadlet( const u_int32_t );

/*! \fn Quadlet( const string& data )
  * \brief Constructs an quadlet from a string.
  * \param data : String representing the data.
  */
  Quadlet( const string& );


/*! \fn Quadlet( const Quadlet& ref )
  * \brief Constructs a copy of ref.
  */
  Quadlet( const Quadlet& );


/*! \fn ~Quadlet()
  * \brief Destructs the object.
  */
  ~Quadlet() {};

//@}

/** \name Manipulate the Quadlet
  *  These functions manipulate the value of a certain range of the Quadlet.
  */

//@{

/*! \fn getByte( const int byte) const
  * \brief This method returns the i'th byte of the quadlet. (value between 0..3)
  * \param byte : The position of the byte.
  */
  u_int32_t getByte( const int ) const;


/*! \fn setByte(const int i, const u_int32_t value)
  * \brief This method sets the i'th byte (0..3) to value
  * \param i : integer i for the position (value between 0..3)
  * \param value : u_int32_t with the data
  */
  void setByte( const int i, const u_int32_t value);


/*! \fn getBitRange( const int start, const int end) const
  * \brief This method returns the bit range from start to end as integer
  * \param start : startvalue of the bitrange.
  * \param end : endvalue of the bitrange.
  */
  u_int32_t getBitRange( const int start, const int end ) const;


/*! \fn setBitRange( const int start, const int end, const u_int32_t value )
  * \brief This method sets the bitrange from start to end to a value.
  *
  * The bitrange must big enough to store the value.
  * \param start : startvalue of the bitrange
  * \param end : endvalue of the bitrange
  * \param value : u_int32_t for the value
  */
  void setBitRange( const int start, const int end, const u_int32_t value );

/*! \fn getBit( const int bit) const
  * \brief This method returns the i'th bit (value between 0..31)
  * \param bit : int with the i'th position
  * \return int : value 0 or 1.
  */
  int getBit( const int ) const;

/*! \fn setBit( const int i, const int value)
  * \brief Sets the i'th bit to value
  * \param i : integer for the position ( value between 0..31)
  * \param value : the i'th bit (0 or 1)
  */
  void setBit( const int i, const int value);
//@}

/** \name Convert a Quadlet
  *  These functions convert a Quadlet to another basic type.
  */

//@{

/*! \fn toInt() const
  * \brief This method converts a quadlet to a 32 bit integer value
  * \return u_int32_t : The integer value of the Quadlet
  */
  u_int32_t toInt() const;

/*! \fn fromInt( u_int32_t value)
  * \brief This method converts a 32 bit integer to a quadlet
  * \param value : value for the Quadlet
  */
  void fromInt( u_int32_t value);

/*! \fn toString() const
  * \brief This method converts a Quadlet to a string
  * \param string : return the Quadlet as string
  */
  string toString() const;

/*! \fn toCharArray ( char *, int ) const
  * \brief converts a Quadlet to a char*
  */
  char *toCharArray( char *, int ) const;

//@}

/** \name Computing operator
  *  These functions implements some computing operators for Quadlets.
  */

//@{
/*! \fn operator=( const Quadlet& q )
  * \brief operator= assigns the data q to this quadlet and returns a reference to it.
  * \param q : Quadlet that will be assigned
  * \return Quadlet : Reference to this Quadlet
  */
  Quadlet& operator=( const Quadlet& );


/*! \fn operator==( const Quadlet& )
  * \brief operator== compares two Quadlets and returns true if they are equal, false if not

  * Two Quadlets are equal, if all 4 bytes are equal.
  * \param Quadlet reference to quadlet
  * \return bool : True if two Quadlets are equal, false if not
  */
  bool operator== ( const Quadlet& );

/*! \fn operator!=( const Quadlet& q)
  * \brief operator!= compares Quadlet q with this Quadlet
  * \param q : Quadlet that should be compared
  * \return bool : return true if q is not equal to this Quadlet, false if not.
  */
  bool operator!= ( const Quadlet& );


/*! \fn operator+( const Quadlet& q_1, const Quadlet& q_2)
  * \brief operator+ adds the value of Quadlet q_1 and q_2 and returns the reference to it.
  *
  *  The new value of q1+q2 must be less then 0xffffffff or the value is set to
  *  0xffffffff.
  * \param q_1 : Reference to the first Quadlet
  *  \param q_2 : Reference the the seconf Quadlet
  * \return const Quadlet : The new Quadlet
  */
  friend const Quadlet operator+( const Quadlet&, const Quadlet& );


/*! \fn operator+( const u_int32_t& data, const Quadlet& q)
  * \brief operator+ adds the value of Quadlet q to this Quadlet and returns the reference to it.
  *  \return Quadlet : The new Quadlet
  */
  friend const Quadlet operator+( const u_int32_t&, const Quadlet&);

/*! \fn operator-( const u_int32_t& data, const Quadlet&)
  * \brief operator+ adds the value of Quadlet q to this Quadlet and returns the reference to it.
  *  \return const Quadlet : The new Quadlet
  */
  friend const Quadlet operator-( const u_int32_t& data, const Quadlet&);

/*! \fn operator-( const Quadlet& q, const Quadlet&)
  * \brief operator- subtract the value of data from this Quadlet and returns the reference to it.
  *  \return const Quadlet : The new Quadlet
  */
  friend const Quadlet operator-( const Quadlet& q, const Quadlet&);
//@}

/*! \fn operator<<( ostream& , const Quadlet&)
  * \brief operator<< writes the Quadlet to a stream and returns a reference to it.
*/
  friend ostream& operator<<( ostream& , const Quadlet&);

/*! \fn  operator>>( istream& , Quadlet&)
  * \brief operator>> reads 4 bytes from a stream, stores the value in the Quadlet and returns a reference to the stream
*/
  friend istream& operator>>( istream& , Quadlet&);

private:

  // these 4 chars contain the quadlet in 'bus byte order', eg
  // the most significant byte in quadlet[0]
  unsigned char quadlet[ 4 ];
  static const int Quadlet_OUTPUTSIZE = 32;
};

}





#endif

