/***************************************************************************
                          l1394_session.h  -  description
                             -------------------
    begin                : Thu Jul 6 2000
    copyright            : (C) 2000-2004 by Michael Repplinger
    email                : repplix@studcs.uni-sb.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef L1394SESSION_H
#define L1394SESSION_H

#include <list>
#include <vector>
#include <string>
#include <iostream>

#include "Thread.hpp"

#include "l1394_vcr.h"
#include "l1394_camera.h"
#include "l1394_card.h"
#include "l1394_csrrom.h"
#include "l1394_resource.h"
#include "l1394_dcccamera.h"
#include "l1394_fcpnode.h"
#include "l1394_nodefactory.h"
#include "l1394_eventhandle.h"
#include "l1394_message.h"
#include "Singleton.hpp"


/*! \mainpage L1394 API documentation
  * \section about About L1394
  * L1394 is a high-level object oriented library to controll and access devices connected
  * to a FireWire (aka IEEE-1394) bus under Linux. Main goal of this library is to hide
  * the generic interfaces of the libraw1394 library and to provide simple and uniform
  * interfaces to access and control the different devices. <BR>
  * The following features are realized:<BR>
  *    - Automatic devices detection : Connected nodes on the bus are detected automaically
  *      and corresponding objects are created to access and control the device.
  *    - Automatic bus rest handling : After a bus reset the internal states of the software
  *      objects are updated automatically.
  *    - Easy event handling : This library provides an easy to used event handling mechanism
  *      to inform the application developer about changes on the FireWire bus via eventlistener.
  *      So the application developer can be informed about new devices or that currently used
  *      devices are no longer available. 
  *    - Well defined interfaces : Every device has a well defined interface for simple control.
  *      In the current version, this library provides unifrom interfaces for Cameras(A/VC and DCC)
  *      and Vcrs.
  *    - Simple control mechanism : To control a device this library provides two objects. The first one access the
  *      feature and the second one defines how to execute a feature.
  *    - Simple access to available devices : The Session object provides functions to access the different devices easily.
  *
  *
  * See <A HREF="../L1394_about.html"> L1394 Homepage</A> for more information.
  *  <BR> <BR>
  *  <HR WIDTH="100%">
  *  \section short_intro Short introduction
  *
  *  The most important object for the application developer is the L1394::Session object.
  *  The Session object is responsible to initialize this library and provides an interface
  *  to access the different devices as well as automatic memeory management.<BR>
  *  Every program using this library needs a single Session object, that can be created
  *  using the the static member function SSession::getSession().
  *  <BR>
  *  The following program shows a little example how to use this library. See
  *  <A HREF="../L1394_tutorial.html">here </A> for a step by step introduction
  *  \code
  *  #include <l1394_session.h>
  *
  *  using namespace L1394;
  *  int main(int argc, char*argv[])
  *  {
  *    //Start a new Session
  *    Session* session = SSession::getSession();
  *
  *    //Search a camera
  *    Camera* camera = session->findCamera();
  *
  *   if (camera == 0) {
  *      cout << "No Camera available" << endl;
  *     return 0;
  *   }
  *
  *    //Now you can use the camera.
  *    //Test if the camera support zooming.
  *    if(camera->zoom()->hasFeature())
  *      //If true set zoom value to the minimum.
  *      camera->zoom()->setValue(
  *          camera->zoom()->getMinValue() )
  *    else
  *      cout << "Zoom is not supported" << endl;
  *
  *    //Before you quit the application, delete the Session
  *    delete session;
  *  }
  *
  *  \endcode
  */

/*! \defgroup L1394_Node
  *  \brief All classes representing a FireWire node are member of this group.
  *
  *  Every node connected to the FireWire bus is represented as a software object.
  *  These software objects are called L1394 nodes. Base class for the different L1394 nodes is
  *  the class L1394::Node.
  *  For information about the different L1394 nodes see the corresponding class documentation.
  */

/*! \defgroup L1394_Device
  * \brief All classes representing interfaces for the different devices are member of this group.
  *
  *  This library provides a well defined interface for all supported devices. Each interface provides
  *  functions to access the different features.
  *  For more information about device specific interfaces see the corresponding class documentation.
  */

/*! \defgroup L1394_Register
  * \brief All classes representing a L1394 Register are member of this group.
  *
  *  If you call a method to execute a feature of a Device a so called L1394 Register object
  *  is returned. The L1394 Register object provides an interface how to execute a command.
  * So you can specify how to execute the actual feature. For information about the different
  * L1394 Register see the corresponding class documentation.
  */

/*! \defgroup L1394_Event
  * \brief All classes for events and event handling are member of this group.
  *
  *  In the actual version of this library, only the classes EventHandle and internal::Event are implemented.
  */


/*! \defgroup L1394_Extensions
  * \brief Member of this group define so called L1394 Modules.
  *
  *  A L1394 module is not part of the raw L1394 library and defines
  * extensions for this library. For more information about a L1394 module
  *  see the specific class documentation.
  */

/*! \defgroup L1394_Display
  * \ingroup L1394_Extensions
  *  \brief L1394_Display module. This module can be used to display the video
  *  content of a device. This module is still under develop.
  */

/*! \defgroup Internal
  * \brief All classes for internal use are member of this group.
  *
  *  Be careful using the CSR-specific classes. They can cause a lot of problems
  * with FireWire cards because The CSR-Rom of the actual FireWire driver is still buggy.
  */

/*! \namespace L1394
  * \brief The base namespace for this library is L1394.
  *
  * This namespace provides all important classes, like L1394 Nodes and L1394 Devices.
  *
  */
namespace L1394{
/*! \class Session
  * \brief The Session class is the most most important class for the programmer.
  *
  * Every program using this library needs exactly one Session object.
  * (There is further now way to create more than one object.)
  *
  * The Session object has two major jobs. The first part is, to instantiate the Card
  * objects and handle incoming events from the driver, like bus-resets. The Card object
  * instantiate the L1394 Nodes for the connected nodes on FireWire bus.<BR>
  * The second one is, to represent a simple interface to find or get the L1394 Nodes for
  *  a FireWire node.  These functions search on all FireWire cards.<BR>
  * There are also some static functions to set some internal parameters, like the
  * output streams, or a so called 'safe-mode'. For more information about these mothods
  *  see the section 'Configure the Library'.
  *
  * Normally your application needs to include the l1394_session.h file, for classes
  * in namespace L1394.<BR>
  * To create a Session object you can call the static member function Session::getSession(), or the macro
  * GetSession();
  *
  * The session object is also responsible to delete all L1394 objects. You must not
  * delete any object from this class, except the Session object itself!! If you delete
  *  some objects, no correct work is possible!!!!
  *
  *  If the getSession() function fails, the library quit the application with an error message.
  *  (The library quits an application only if the getSession() function fails)
  *
  *  A little example:
  *  \code
  *  #include <l1394_session.h>
  *
  *  using namespace L1394;
  *
  *  int main(int argc, char* argv[])
  *  {
  *    //Start a new Session.
  *    Session *session = GetSession();
  *
  *    for (int i = 0; i< session->getCardCount(); i++)
  *      session->getCard(i)->printNodeList();  //prints the device list to cout
  *
  *    //Get all cameras
  *    list<Camera*> camera_list = session->getAllCameras();
  *    list<Camera*>::iterator camera_iterator = camera_list.begin();
  *
  *    // Now you can work with the cameras. For example calibrate all cameras.
  *    for (int i = 0; i< camera_list.size(); i++)
  *    {
  *      if ((*camera_iterator)->whiteBalance()->hasFeature())
  *        (*camera_iterator)->whiteBalance()->onePush();
  *      if ((*camera_iterator)->zoom()->hasFeature())
  *        (*camera_iterator)->zoom()->setValue((*camera_iterator)->zoom()->getMinValue));
  *      //and so on ....
  *    }
  *
  *    //Get all vcrs
  *    list<Vcr*> vcr_list = session->getAllVcrs();
  *    list<Vcr*>::iterator vcr_iterator = vcr_iterator.begin();
  *
  *    //Now you can work with the devices.
  *    for (int i = 0; i< vcr_list.size(); i++)
  *    {
  *      //start all vcrs
  *      (*vcr_iterator)->play()->send();
  *      vcr_iterator++;
  *    }
  *
  *  }
  *  \endcode
  *
  * For more information see the tutorial.
  *
  * @author Michael Repplinger
  */

/*! \namespace internal
  * \brief Namespace for internal classes
  *
  * The internal namespace provides all classes and structures, for internal use. These classes
  * can be used to get detailed information about the FireWire bus and the connected nodes.
  */
class Card;
class FcpNode;

class Session
{
friend class L1394::CreateUsingNew<Session>;
public:

/*! \name Access the FireWire bus
  * These functions search and return the different nodes on the FireWire bus.
  *  All functions search on all installed FireWire cards.
  */
//@{
/*! \fn getCardCount() const
 *  \brief This method returns the number of cards in your computer.
 *  \return int : integer with the card count.
 */
  inline int getCardCount()                              const { return card_count; }

  inline const std::vector<std::string>& getIsoDevices() const { return __iso_devices; }

/*! \fn addEventHandle(EventHandle*)
  * \brief An EventHandle object added with this method reports about busresets on all cards
  */
  void addEventHandle(EventHandle*)          ;


/*! \fn removeEventHandle(EventHandle*)
  * \brief This method removes an EventHandle
  */
  void removeEventHandle(EventHandle*)  ;


/*! \fn nodeExist(u_int64_t guid) const
 *  \brief This method tests, if a node with a specific guid exist.
 *
 *  This method searches on all cards.
 *  \param guid : u_int64_t for the guid of the node.
 *  \return bool : true if a node exist, else false.
 */
  bool nodeExist(u_int64_t guid) const;


/*! \fn getCard(unsigned int card_id) const
 *  \brief This method returns a pointer to the i'th card (beginning by 0).
 *
 *  \param card_id : the current card-id (value between 0..15).
 *  \return Card*  : pointer to the Card object, if a card with this id exist, else NULL.
 */
  const Card* getCard(unsigned int card_id)  const;


/*! \fn getCardList()  const
  * \brief This method returns the card_array.
  *
  *  The card_array has size 16 and stores the pointer to all Card objects.
  * If less than 16 cards the array is filled with NULL.
  * \return list<const Card*> : Pointer to an array of size 16 with Card objects.
  */
  list<const Card*> getCardList()              const;


/*! \fn findDevice(const string& name) const
  * \brief This method finds the first Camera and returns it.
  *
  * This method searches on all Card objects.
  * \return Device* : pointer to the first Camera found on a card, NULL if no Camera exist.
  */
  Device* findDevice(const string& name = "Device")  const;

/*! \fn findDevice(u_int64_t guid)
  * \brief This method finds the Device with GUID guid.
  *
  * This method searches on all Card objects.
  * \return Device* : pointer to the device with GUID guid, NULL if no such Device exist.
  */
  Device* findDevice(u_int64_t guid);

/*! \fn findCamera(u_int64_t guid)
  * \brief This method finds the Camera with GUID guid.
  *
  * This method searches on all Card objects.
  * \return Camera* : pointer to the Camera with GUID guid, NULL if no such Camera exist.
  */
  Camera* findCamera(u_int64_t guid);

/*! \fn findVcr(u_int64_t guid)
  * \brief This method finds the Device with GUID guid.
  *
  * This method searches on all Card objects.
  * \return Vcr* : pointer to the Vcr with GUID guid, NULL if no such Device exist.
  */
  Vcr* findVcr(u_int64_t guid);


/*! \fn findCamera(const string& name) const
  * \brief This method finds the first Camera and returns it.
  *
  * This method searches on all Card objects.
  * \return Camera* : pointer to the first Camera found on a card, NULL if no Camera exist.
  */
  Camera* findCamera(const string& name = "Camera")  const;


/*! \fn findVcr(const string& name) const
  * \brief This method finds a Vcr and returns a pointer to it.
  * \return Vcr* : pointer to the first vcr found, NULL if no Vcr exist.
  */
  Vcr* findVcr(const string& name = "Vcr")            const;


/*! \fn findAvcVcr(const string& name) const
  * \brief This method finds an AvcVcr and returns a pointer to it.
  * \return Vcr* : pointer to the first AvcVcr found, NULL if no AvcVcr exist.
  */
  AvcVcr* findAvcVcr(const string& name = "AvcVcr")   const;


/*! \fn findDccCamera(const string& name) const
  * \brief This method finds the first DccCamera on FireWire bus and return it.
  * \return DccCamera* : pointer to the first DccCamera, NULL if no DccCamera exist.
  */
  DccCamera* findDccCamera(const string& name = "DccCamera")  const;


/*! \fn findAvcCamera(const string& name) const
  * \brief This method find the first AvcCamera on the FireWire bus.
  * \return AvcCamera* : pointer to the first AvcCamera, NULL if no AvcCamera exist.
  */
  AvcCamera* findAvcCamera(const string& name = "AvcCamera")  const;


/*! \fn getAllDevices() const
  * \brief This method returns a list of all Cameras on the FireWire bus.
  *
  * This method searches on all Cards
  * \return list<Camera*> : returns a stl-list with pointer to all Cameras.
  */
  list<Device*> getAllDevices()          const;


/*! \fn getAllCameras() const
  * \brief This method returns a list of all Cameras on the FireWire bus.
  *
  * This method searches on all Cards
  * \return list<Camera*> : returns a stl-list with pointer to all Cameras.
  */
  list<Camera*> getAllCameras()          const;


/*! \fn getAllVcrs() const
  * \brief This method returns a list with all Vcrs on the FireWire bus.
  * \return list<Vcr*> : return a stl-list with pointer to all Vcrs.
  */
  list<Vcr*> getAllVcrs()                const;


/*! \fn getAllAvcVcrs() const
  * \brief This method returns a list with all AvcVcrs on the FireWire bus.
  * \return list<AvcVcr*> : return a stl-list with pointer to all AvcVcrs .
  */
  list<AvcVcr*> getAllAvcVcrs()          const;


/*! \fn getAllDccCameras() const
  * \brief This method finds all DccCameras and return a list of them.
  * \return list<DccCamera*> :a stl-list with pointer to all DccCameras.
  */
  list<DccCamera*> getAllDccCameras()    const;


/*! \fn getAllAvcCameras() const
  * \brief This method finds all AvcCameras and return a list of them.
  * \return list<AvcCamera*> :a stl-list with pointer to all AvcCameras.
  */
  list<AvcCamera*> getAllAvcCameras()    const;

//@}


/*! \name Configure the library
  * These functions set some internal parameters. This library uses three ostreams
  * to print messages. The debug_stream prints debug_messages, the message_stream
  * prints some more or less informative messages and the error_stream prints error
  * messages. By default all streams are set to cout. To disable the messages, set
  *  the stream to NULL.
  */

//@{
/*! \fn setDebugStream(std::ostream* output)
  * \brief This method sets the debug output.
  *
  *  \param output : pointer to the output for debug messages.
  */
  static void setDebugStream(std::ostream* o)     { internal::SMessage::getInstance()->setDebugStream(o);}

/*! \fn setMessageStream(std::ostream* output)
  * \brief This method sets the standard output.
  *
  *  \param output : pointer to the output for debug messages.
  */
  static void setMessageStream(std::ostream* o)  { internal::SMessage::getInstance()->setMessageStream(o);}


/*! \fn setErrorStream(std::ostream* output)
  * \brief This method sets the error output.
  *
  *  \param output : pointer to the output for debug messages.
  */
  static void setErrorStream(std::ostream* o)    { internal::SMessage::getInstance()->setErrorStream(o);}

/*! \fn setWarningStream(std::ostream* output)
  * \brief This method sets the error output.
  *
  *  \param output : pointer to the output for debug messages.
  */
  static void setWarningStream(std::ostream* o)  { internal::SMessage::getInstance()->setWarningStream(o);}

/*! \fn safeMode(bool mode , int delay)
 *  \brief This method enables the safe-mode for the read and write functions.
 *
 *  By enabling safe mode the read, write and lock functions are
 *  delayed for some msecs. If you send a lot of asynchrone commands, a bus rest
 *  can occur.
 *  \param mode : true enables safe-mode, false disables safe-mode
 *  \param delay : how many msecs delayed between asynchronous commands.
 */
  static void safeMode(bool mode = false, int delay = 30){internal::Transaction::setSafeMode(mode, delay);}
//@}

protected:
/*! \name Session constructor
  * These functions creates the Session objects.
  */
//@{
/*! \fn getSession()
 *  \brief This method returns a pointer to the Session object.
 *
 *  If no session object exist, it is created.
 *  \return Session* : pointer to the Session object.
 */
  static Session* getSession();


/*! \fn Session()
 *  \brief Init a Session.
 */
  Session();

  virtual ~Session();
//@}

/*! \fn getNode(u_int64_t guid)   const
 *  \brief This method returns a pointer to a node specified by its guid return NULL, if node doesn't exist.
 *
 *  This method searches on all cards.
 *  \param guid : u_int64_t value for the guid.
 *  \return Node* : pointer to the Node, if a node with this guid exist, else NULL.
 */
  Node* getNode(u_int64_t guid)    const;


/*! \fn findNode(int node_type) const
 *  \brief finds a node, specified by it's node_type.
 *
 *   A pointer to the first found node was returned, NULL if no node with this type was found.
 *  \param node_type : value that specify a node-type, defined in class Node.
 */
  Node* findNode(int node_type) const;


/*! \fn getAllNodes(int node_type) const
 *  \brief returns a list with Nodes from type node_type
 *  \param node_type : the node type
 */
  list<Node*> getAllNodes(int node_type) const;

private: //method
  void allocate();
  void deallocate();
  void initCard(int);
  void scanIsoDevices();

private: //member
  raw1394handle_t  session_default_handle, tmp_handle;
  raw1394_portinfo pinf[16];

  int card_count;

  Card **card_array;

  std::vector<std::string> __iso_devices;

  static int  busResetHandler(raw1394handle_t h, unsigned int);
  static void *startBusReset(void*);

  raw1394handle_t *reset_handle_array;
  raw1394handle_t *default_handle_array;
  Thread          *thread_array;
  internal::Message* message;
public:  
  static bool        session_is_running;
  static ThreadMutex global_mutex;
};

typedef  Singleton<L1394::Session, CreateUsingNew> SSession;

}
#endif
