/***************************************************************************
                          l1394_transaction.h  -  description
                             -------------------
    begin                : Sun Jul 1 2001
    copyright            : (C) 2001-2004 by Michael Repplinger
    email                : repplix@studcs.uni-sb.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef L1394_TRANSACTION_H
#define L1394_TRANSACTION_H
#include <netinet/in.h>
#include <sys/poll.h>
#include <sys/types.h>
#include <sys/time.h>
#include <unistd.h>
#include <string>
#include "libraw1394/csr.h"
#include "libraw1394/raw1394.h"

#include "l1394_quadlet.h"
#include "l1394_qarray.h"
#include "l1394_resource.h"
#include "l1394_message.h"

namespace L1394 {
namespace internal {


/** \class Transaction
  * \brief This class is a wrapper class for the libraw1394 library.
  *
  * It provides methods for the FireWire specific transactions (read, write and lock)
  * and AV/C specific transactions (send).
  *
  *@author Michael Repplinger
  */
class Transaction {
public:

/*! \name Transaction constructor
  * These functions creates a Transaction object.
  */
//@{
/*! \fn Transaction(raw1394handle_t)
  * \brief Constructor
  */
  Transaction(raw1394handle_t);


/*! \fn ~Transaction()
  * \brief Destructor
  */
  ~Transaction();
//@}


/*! \name Transactions
  * These methods are wrapper for the libraw functions.
  */
//@{
/*! \fn  read(const u_int64_t address, Quadlet* quadlet, const u_int32_t node_id) const
  * \brief This method reads 4 bytes from CsrRom of a specific node and return the value.
  * \param address : u_int64_t integer value for the address of CSR
  * \param node_number : send to node with node_number
  *  \param quadlet : pointer to the Quadlet to store the value.
  *  \return Quadlet : value stored in this address
  */
  int read(const u_int64_t address, Quadlet* q, const u_int32_t node_id) const;


/*! \fn  write(const u_int64_t address, const Quadlet& data, const u_int32_t node_id) const
  * \brief This method writes 4 bytes to CsrRom of a specific node
  * \param address : u_int64_t value for the CSR-address
  * \param data : Quadlet with the data that should be written to CSR-address
  * \param node_number : write to node with this actual node_number
  */
  int write(const u_int64_t address, const Quadlet& data, const u_int32_t node_id ) const ;


/*! \fn lock   (const u_int64_t address, const Quadlet& data, const unsigned int extcode,const Quadlet& argument, Quadlet* result, const u_int32_t node_id) const
  * \brief This method realizes an atomic transaction on the FireWire bus.
  *
  * An atomic transaction means, that no node can access the address during the lock operation.
  * \param address : u_int64_t value for the CSR-address
  *  \param data : The data Quadlet defines the value that should be executed on the address
  * \param extcode : The extension code defines the type of lock operation.
  *  \param argument : The argument value is compared with the actual address value and depending on the extension code a operation with the data value is executed.
  *  \param result : If a value is returned, it is stored in result
  * \param node_id : current node-id of the node
  *  \return int: L1394_FAILED, if the transaction fails, else L1394_SUCCESS.
  */
  int lock(const u_int64_t address,const Quadlet& data,const unsigned int extcode, const Quadlet& argument, Quadlet* result, const u_int32_t node_id) const  ;


/*! \fn send(const Quadlet& command, const u_int32_t node_id) const
  * \brief This method sends a Quadlet (4 bytes) to the node with current node_ide.
  * \param command  : The Quadlet to send.
  * \param node_id  : current node_id of the node
  * \return Quadlet : The Quadlet with response code.
  */
  Quadlet send(const Quadlet& command, const u_int32_t node_id) const;


/*! \fn send(const QArray& command, const u_int32_t node_id) const
  * \brief This method sends an QArray to a node.
  * \param QArray with the extended command
  * \param node_id : current node_id of the node
  * \return QArray with the response code.
  */
  QArray  send(const QArray&  command, const u_int32_t node_id) const;
//@}

/*! \fn setSafeMode(bool b, int delay)
  * \brief This method enables/disables the safe mode for asynchrone transactions.
  *
  * If you enable the safemode a delay between every transaction is enabled.
  */
  static  void setSafeMode(bool b, int d) {safe_mode = b; delay_value = d;}

  //! delay for msecs i
  static void delay(int i);

private:

  static int L1394FcpHandler(raw1394handle_t , nodeid_t nodeid, int , size_t length, unsigned char *data);

  static bool safe_mode;
  static int  delay_value;

  static QArray* response_block;

  raw1394handle_t default_handle;
  //static int my_tag_handler(raw1394handle_t handle, unsigned long tag,
  //                 raw1394_errcode_t errcode);

};

} //end namespace internal
} //end namespace L1394
#endif
