/***************************************************************************
                          vcr.h  -  description
                             -------------------
    begin                : Fri Nov 24 2000
    copyright            : (C) 2000-2004 by Michael Repplinger
    email                : repplinger@cs.uni-sb.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef VCR_H
#define VCR_H

#include "l1394_deviceregister.h"
#include "l1394_eventhandle.h"
#include "l1394_device.h"
#include "l1394_timecode.h"
#define EXECUTE 1
#define TEST 0

namespace L1394
{

enum{
  DVCR_SPEED_32 = 0,
  DVCR_SPPED_STANDARD,
  DVCR_SPPED_176,
  DVCR_192,

  DVHS_SPEED_STANDARD,

  VHS_SPEED_33,
  VHS_SPEED_32,
  VHS_SPEED_STANDARD

  };

}//end namespace

namespace L1394
{
/*! \class Vcr
  *  \ingroup L1394_Device
  * \brief This class defines the interface for Vcrs.
  *
  * This class is base-class for all Vcrs. The interface provides functions
  *  for the most common features. If you access a feature of the Vcr object
  *  the DeviceRegister object is returned and you can specify how to execute the command.
  * (See also the DeviceRegister class documentation.)
  * A little example show the idea.
  *
  * \code
  *  Session *session = GetSession();
  * Vcr* my_vcr = session->findVcr();
  * if( my_vcr->fastForward()->hasFeature() )
  * {
  *   int response = my_vcr->forward()->send();
  *   switch(response) //for all response codes see method send() in class DeviceRegister.
  *   {
  *     case ACCEPTED : cout << "Command accepted" << endl; break;
  *     case REJECTED : cout << "Command rejected" << endl; break;
  *     default : cout << "Unhandled case" << endl; break;
  *   }
  * }
  * \endcode
  *@author Michael Repplinger
  */

class Vcr : public Device {

public:

  Vcr(Node* parent_node);

/*! \fn ~Vcr()
  * \brief destructor
  */
  virtual ~Vcr(){};

/*! \name Vcr Interface
  * This functions describe the interface of any vcr
  */
//@{
  virtual DeviceType getDeviceType() const {return _Vcr;}

/*! \fn play()
  * \brief With this method you can access the play feature.
  * \return DeviceRegister* : pointer to the DeviceRegister.
  */
  virtual int play()                    =0;


/*! \fn pause()
  * \brief With this method you can access the pause feature.
  * \return DeviceRegister* : pointer to the DeviceRegister.
  */
  virtual int pause()                   =0;

/*! \fn timeCode getTimeCode()
  * \brief With this method you can get the actual timecode.
  * \return Timecode : the actual timecode.
  */
  virtual Timecode getTimeCode()                       =0;

/*! \fn recordingDate(const Timecode time_begin)
  * \brief With this method you can access the recording date feature.
  * \return DeviceRegister* : pointer to the DeviceRegister object.
  */
  virtual const DeviceRegister* recordingDate(const Timecode time_begin)   =0;


/*! \fn transportState()
  * \brief With this method you can get the actual status of a vcr.
  * \return DeviceRegister* : pointer to the DeviceRegister object.
  */
   virtual const DeviceRegister* transportState()   =0;


/*! \fn search(const Timecode)
  * \brief This method stop the tape at a specific position.
  * \return DeviceRegister* : pointer to the DeviceRegister.
  */
  virtual const DeviceRegister* search(const Timecode)   =0;


/*! \fn nextFrame()
  * \brief With this method you can access the nextFrame feature.
  *
  * If you execute the command, the vcr increase one frame, from actual
  * position.
  * \return DeviceRegister* : pointer to the DeviceRegister.
  */
  virtual const DeviceRegister* nextFrame()               =0;

/*! \fn previousFrame()
  * \brief With this method you can access the nextFrame feature.
  *
  * If you execute the command, the vcr decrease one frame, from actual
  * position.
  * \return DeviceRegister* : pointer to the DeviceRegister.
  */
  virtual const DeviceRegister* previousFrame()           =0;


/*! \fn slowForward()
  * \brief With this method you can access the slowForward feature.
  * \return DeviceRegister* : pointer to the DeviceRegister.
  */
  virtual const DeviceRegister* slowForward()             =0;


/*! \fn slowReverse()
  * \brief With this method you can access the slowReverse feature.
  * \return DeviceRegister* : pointer to the DeviceRegister.
  */
  virtual const DeviceRegister* slowReverse()             =0;


/*! \fn forward()
  * \brief With this method you can access the forward feature.
  * \return DeviceRegister* : pointer to the DeviceRegister.
  */
  virtual const DeviceRegister* forward()                 =0;


/*! \fn reverse()
  * \brief With this method you can access the reverse feature.
  * \return DeviceRegister* : pointer to the DeviceRegister.
  */
  virtual const DeviceRegister* reverse()                 =0;


/*! \fn fastForward()
  * \brief With this method you can access the fastForward feature.
  * \return DeviceRegister* : pointer to the DeviceRegister.
  */
  virtual const DeviceRegister* fastForward()             =0;


/*! \fn rewind()
  * \brief With this method you can access the rewind feature.
  * \return DeviceRegister* : pointer to the DeviceRegister.
  */
  virtual const DeviceRegister* rewind()                  =0;


/*! \fn record()
  * \brief With this method you can access the record feature.
  * \return DeviceRegister* : pointer to the DeviceRegister.
  */
  virtual const DeviceRegister* record()                  =0;


/*! \fn recordingSpeed(const int speed)
  * \brief With this method you can access the recordingSpeed feature.
  * \param speed : the speed value.
  * \return DeviceRegister* : pointer to the DeviceRegister.
  */
  virtual const DeviceRegister* recordingSpeed(const int speed)   =0;
//@}
};
}
#endif
