/***************************************************************************
                          l1394_window.h  -  description
                             -------------------
    begin                : Thu Nov 30 2000
    copyright            : (C) 2000-2004 by Michael Repplinger
    email                : repplinger@cs.uni-sb.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef L1394_MM_WINDOW_H
#define L1394_MM_WINDOW_H
#include "l1394_converter.h"
#include <l1394_session.h>

namespace L1394_MM{

/*! \class Window
  *  \ingroup L1394_Display
  * \brief This abstract class represents the interface for all windows.
  *
  * If you want to use your own window classes with this library, you inherit
  * the new window class from this class and implement the abstract functions.
  *  For Example see class XShmImg.
  *
  *@author Michael Repplinger
  */

class Window : protected L1394::EventHandle {
public:


/*! \  fn Window()
  * \ brief constructor
  */
  Window(l1394_converter dec);

/*! \fn ~Window()
  * \brief destructor
  */
  virtual  ~Window();


/*! \fn getBuffer() const
  * \brief This function return the window buffer to write data in it.
  *
  * If the window has no own buffer, the assigned buffer will be returned, if it is
  * set, if not NULL is returned.
  * \return char* : pointer to buffer, represent as char array.
  */
  virtual unsigned char* getBuffer() const =0;


/*! \fn void setTitle(const char* title)
  * \brief With this function you can set the title of window.
  * \param title : const char* with the name of the window
  */
  virtual void setTitle(const char* title) = 0;


/*! \fn void update()
  * \brief This function update the display.
  *
  * This function must be called to show changes on the buffer.
  */
  virtual void update() = 0;


/*! \fn getColorDepth() const
  * \brief This function return the color depth of the window.
  *
  * This value depends on your X-Server settings.
  * \return int : the color depth as integer value, normally 16 or 24
  */
  virtual int getColorDepth() const =0;


/*! \fn start()
  * \brief This function starts the converter and diplay the frames.
  *
  *  You don't need to reimplement this function if you use the run_static function
  *  to run the window.
  */
  virtual void start();


/*! \fn stop()
  * \brief This function stops the running window.
  */
  virtual void stop();


/*! \fn connect(L1394::Device* device)
  * \brief This function connects an L1394::Device to this window.
  *
  *  This class inherits from class EventHandle and adds themself to
  *  the device eventhandling. If the device is deleted this object delete
  * themself and you must not delete the window object.
  * \param device: pointer to the device.
  */
  virtual void connect(L1394::Device*);

/** Disconnects a previous connected device, so the window can be reused by
  * another device
  */
  virtual void disconnect();

/** Returns true, if the window is started, else false
  */
  bool isRunning() const {return   running;}
protected:


  L1394::Device* getDevice() const {return device;}
  L1394::internal::Message* message;

private:
  //! This function create a thread with function initWindow();
  static void* run_static(void*);

  //! This function gets a frame from the device, converts it and updates the window.
  virtual void* initWindow();


  //! In this thread runs the initWindow() function.
  L1394::Thread window_thread;

  //! While running is true, the initWindow() function is running
  bool running;

  //! The converter function
  l1394_converter conv;

  //! The connected device
  L1394::Device* device;

  L1394::ThreadMutex mutex;
};

}
#endif
